"""
@Author: kang.yang
@Date: 2024/10/8 15:04
"""
from kytest.hm import Page, Elem


class HmPage(Page):
    my_entry = Elem(text='我的')
    login_entry = Elem(text='登录/注册')
    pwd_login = Elem(text='账号登录')
    forget_pwd = Elem(text='忘记密码')




import time

from kytest.utils.log import logger
from kytest.hm.driver import HmDriver


class Elem(object):
    """
    鸿蒙元素定义
    https://github.com/codematrixer/hmdriver2
    """

    def __init__(self,
                 driver: HmDriver = None,
                 **kwargs):
        """
        @param driver: 安卓驱动
        @param watch: 需要处理的异常弹窗定位方式列表
        """
        self._driver = driver
        self._watch = kwargs.pop('watch', None)
        self._kwargs = kwargs

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # 公共方法
    def _match(self, text):
        """
        判断元素是否存在当前页面
        @return:
        """
        _element = self._driver.d(text=text)
        result = text if _element.exists(timeout=0.1) else None
        return result

    def pop_check(self, timeout=3):
        logger.info(f"开始弹窗检测: {self._watch}")
        # 多线程match，如果match到，获取第一个非None内容，进行点击
        # match完休息1s，如果休息3s也没有match到，就停止（定义一个flag，match到就清零）
        # 如果3s内仍然能match到就继续（如果flag大于3就停止）
        _build_info = ["允许", "使用App时允许", "始终允许", "同意"]
        if self._watch is True:
            loc_list = _build_info
        else:
            loc_list = list(set(_build_info + self._watch))
        flag = timeout
        while flag > 0:
            import concurrent.futures

            logger.info(f"匹配: {loc_list}")
            with concurrent.futures.ThreadPoolExecutor() as executor:
                results = executor.map(self._match, loc_list)
                results = [item for item in results if item is not None]

            if results:
                logger.info(f"匹配到: {results}")
                self._driver.d(text=results[0]).click()
                logger.info("点击成功")
                flag = timeout
            else:
                logger.info("匹配失败")

            flag -= 1
            time.sleep(1)
        logger.info("结束检测")

    def find(self, timeout=5, n=3):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @param n：失败后重试的次数
        @return:
        """
        logger.info(f"查找: {self._kwargs}")
        _element = self._driver.d(**self._kwargs)

        if self._watch:
            self.pop_check()

        retry_count = n
        if _element.find_component(wait_time=timeout):
            logger.info(f"查找成功")
            return _element
        else:
            if retry_count > 0:
                for count in range(1, retry_count + 1):
                    logger.info(f"第{count}次重试...")
                    if self._watch:
                        self.pop_check()
                    if _element.find_component(wait_time=timeout):
                        logger.info(f"查找成功")
                        return _element

            logger.info("查找失败")
            self._driver.shot("查找失败")
            raise Exception(f"控件: {self._kwargs}, 查找失败")

    def exists(self, timeout=5):
        logger.info("是否存在")
        return self._driver.d(**self._kwargs).exists(wait_time=timeout, retries=0)

    def info(self):
        logger.info("获取控件信息")
        return self.find().info

    def count(self):
        logger.info("获取控件数量")
        return self.find().count

    def get_text(self, timeout=5):
        logger.info("获取文本")
        return self.find(timeout=timeout).text

    def click(self, timeout=5):
        logger.info("点击")
        self.find(timeout=timeout).click()

    def double_click(self, timeout=5):
        logger.info("双击")
        self.find(timeout=timeout).double_click()

    def long_click(self, timeout=5):
        logger.info("长按")
        self.find(timeout=timeout).long_click()

    def drag_to(self, timeout=5, *args, **kwargs):
        logger.info("拖动")
        self.find(timeout=timeout).drag_to(*args, **kwargs)

    def pinch_in(self, timeout=5, *args, **kwargs):
        logger.info("缩小")
        self.find(timeout=timeout).pinch_in(*args, **kwargs)

    def pinch_out(self, timeout=5, *args, **kwargs):
        logger.info("放大")
        self.find(timeout=timeout).pinch_out(*args, **kwargs)

    def input(self, text, timeout=5):
        logger.info("输入")
        self.find(timeout=timeout).input_text(text)

    def clear(self, timeout=5, *args, **kwargs):
        logger.info("清空")
        self.find(timeout=timeout).clear_text(*args, **kwargs)

    def assert_exists(self, timeout=5):
        logger.info("断言控件存在")
        assert self.exists(timeout=timeout)

    def assert_text_eq(self, text, timeout=5):
        logger.info(f"断言控件文本等于: {text}")
        assert text == self.get_text(timeout=timeout)

    def assert_text_ct(self, text, timeout=5):
        logger.info(f"断言控件文本包括: {text}")
        assert text in self.get_text(timeout=timeout)


if __name__ == '__main__':
    pass










"""
@Author: kang.yang
@Date: 2024/9/24 14:13
"""
# 需要启动tms服务
from kytest.api import generate_case


if __name__ == '__main__':
    host = "http://localhost:8001"
    project_name = 'kz-bff-patent'
    controller = 'Patent Controller'
    generate_case(host, project_name, controller)



import time
import threading

from .driver import Driver

from kytest.utils.log import logger


lock = threading.Lock()


# 链式调用临时存储定位方式
class ChildLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'child'


class Elem(object):
    """
    IOS原生元素定义
    https://github.com/openatx/facebook-wda
    """

    def __init__(self, driver: Driver = None, watch: list = None, **kwargs):
        """
        @param driver: IOS驱动
        @param watch: 异常弹窗处理定位方式列表
        """
        self._kwargs = kwargs
        self._driver = driver
        self._xpath = kwargs.get('xpath', None)
        self._watch = watch
        self.locators = []

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # 链式调用方法
    def child(self, *args, **kwargs):
        self.locators.append(ChildLocator(*args, **kwargs))
        return self

    def _match(self, text):
        """
        判断元素是否存在当前页面
        @return:
        """
        _element = self._driver.d(text=text)
        result = text if _element.wait(timeout=0.1,
                                       raise_error=False) else None
        return result

    def pop_check(self, timeout=3):
        logger.info(f"开始弹窗检测: {self._watch}")
        # 多线程match，如果match到，获取第一个非None内容，进行点击
        # match完休息1s，如果休息3s也没有match到，就停止（定义一个flag，match到就清零）
        # 如果3s内仍然能match到就继续（如果flag大于3就停止）
        _build_info = ["允许", "使用App时允许", "始终允许", "同意"]
        if self._watch is True:
            loc_list = _build_info
        else:
            loc_list = list(set(_build_info + self._watch))
        flag = timeout
        while flag > 0:
            import concurrent.futures

            logger.info(f"匹配: {loc_list}")
            with concurrent.futures.ThreadPoolExecutor() as executor:
                results = executor.map(self._match, loc_list)
                results = [item for item in results if item is not None]

            if results:
                logger.info(f"匹配到: {results}")
                self._driver.d(text=results[0]).click()
                logger.info("点击成功")
                flag = timeout
            else:
                logger.info("匹配失败")

            flag -= 1
            time.sleep(1)
        logger.info("结束检测")

    def find(self, timeout=5):
        """
        针对元素定位失败的情况，抛出KError异常
        @param timeout:
        ["允许", "同意"]
        @return:
        """
        logger.info(f"查找: {self._kwargs}")
        # 元素定义
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath else self._driver.d(**self._kwargs)

        # 链式调用方法叠加
        if self.locators:
            logger.info(f"链式调用: {self.locators}")
            for loc_obj in self.locators:
                if loc_obj.type == "child":
                    _element = _element.child(*loc_obj.args, **loc_obj.kwargs)

        if self._watch:
            self.pop_check()

        # 定位过程
        retry_count = 3
        try:
            if _element.wait(timeout=timeout):
                logger.info(f"查找成功")
                return _element
            else:
                for count in range(1, retry_count + 1):
                    logger.info(f"定位失败第{count}次重试...")
                    if self._watch:
                        self.pop_check()
                    if _element.wait(timeout=timeout):
                        logger.info(f"查找成功")
                        return _element

                logger.info("查找失败")
                self._driver.shot("查找失败")
                raise Exception(f"控件: {self._kwargs}, 查找失败")
        except ConnectionError:
            logger.info('wda连接失败, 进行重连!!!')
            self._driver.util.start_wda()

            for count in range(1, retry_count + 1):
                logger.info(f"连接失败第{count}次重试...")
                if self._watch:
                    self.pop_check()
                if _element.wait(timeout=timeout):
                    logger.info(f"查找成功")
                    return _element

            logger.info("查找失败")
            self._driver.shot("查找失败")
            raise Exception(f"控件: {self._kwargs}, 查找失败")

    def get_text(self, timeout=5):
        logger.info(f"获取文本")
        return self.find(timeout=timeout).text

    def exists(self, timeout=5):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f"是否存在")
        result = False
        try:
            _element = self.find(timeout=timeout)
            result = True
        except:
            result = False
        finally:
            logger.info(result)
            return result

    def _adapt_center(self, timeout=5):
        """
        修正控件中心坐标
        """
        bounds = self.find(timeout=timeout).bounds
        left_top_x, left_top_y, width, height = \
            bounds.x, bounds.y, bounds.width, bounds.height
        center_x = int(left_top_x + width/2)
        center_y = int(left_top_y + height/2)
        logger.info(f'{center_x}, {center_y}')
        return center_x, center_y

    def click(self, timeout=5):
        """
        单击
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        """
        logger.info(f'点击')
        x, y = self._adapt_center(timeout=timeout)
        self._driver.d.appium_settings({"snapshotMaxDepth": 0})
        self._driver.d.tap(x, y)
        self._driver.d.appium_settings({"snapshotMaxDepth": 50})

    def clear(self, timeout=5):
        """清除文本"""
        logger.info("清空")
        self.find(timeout=timeout).clear_text()

    def input(self, text, timeout=5):
        """输入内容"""
        logger.info(f"输入")
        self.find(timeout=timeout).set_text(text)

    def assert_exists(self, timeout=5):
        logger.info("断言控件存在")
        assert self.exists(timeout=timeout)

    def assert_text_eq(self, text, timeout=5):
        logger.info(f"断言控件文本等于: {text}")
        assert text == self.get_text(timeout=timeout)

    def assert_text_ct(self, text, timeout=5):
        logger.info(f"断言控件文本包括: {text}")
        assert text in self.get_text(timeout=timeout)


if __name__ == '__main__':
    pass




















import json
import os.path
import time

from urllib.parse import urlparse
from allure import feature as project, story, title, step
# 这里的project和runner中的project不能同时使用，不然报告中会出现重复用例


class AllureData:
    """解析allure_results的数据"""

    def __init__(self, result_path='report'):
        self.result_path = result_path

    def get_files(self):
        """获取以result.json结尾的文件列表"""
        file_list = []
        for filename in os.listdir(self.result_path):
            if filename.endswith('result.json'):
                file_list.append(filename)

        if not file_list:
            raise KeyError('报告数据为空')

        return [os.path.join(self.result_path, item)
                for item in os.listdir(self.result_path)
                if item.endswith('result.json')]

    @staticmethod
    def get_file_content(file_path):
        """获取文件内容并转成json"""
        with open(file_path, 'r', encoding='UTF-8') as f:
            content = json.load(f)
        return content

    def parser_content(self, content):
        """解析执行结果"""
        project = '未知项目'
        module = '未知模块'
        labels = content["labels"]
        for label in labels:
            if label['name'] == 'feature':
                project = label['value']
            if label['name'] == 'story':
                module = label['value']

        name = content['name']
        full_name = content['fullName']
        parameters = content.get('parameters', [])

        status = content['status']
        log_path = ''
        screen_path = []
        attachments = content['attachments']
        for attach in attachments:
            if attach['type'] == 'text/plain' and attach['name'] == 'log':
                log_path = os.path.join(self.result_path, attach['source'])
            if attach['type'] == 'image/png':
                screen_path.append(os.path.join(self.result_path, attach['source']))

        interfaces = []
        if log_path:
            log_content = open(log_path, 'r', encoding='utf8').read()
            for line in log_content.split("\n"):
                if 'url]: ' in line:
                    interface = line.strip().split('url]: ')[1]
                    parsed_url = urlparse(interface)
                    path = parsed_url.path
                    method = line.strip().split('[method]: ')[1].split()[0].lower()
                    interfaces.append((method, path))

        start = content.get('start')
        end = content.get('stop')
        start_time, end_time = time.strftime("%Y-%m-%d %H:%M:%S",
                                             time.localtime(start / 1000)), \
                               time.strftime("%Y-%m-%d %H:%M:%S",
                                             time.localtime(end / 1000))
        cost = (end - start) / 1000
        if cost > 60:
            cost = '{}min'.format(round(cost / 60, 1))
        else:
            cost = '{}s'.format(round((end - start) / 1000, 1))

        case_data = {
            "project": project,
            "module": module,
            "name": name,
            "full_name": full_name,
            "parameters": parameters,
            "status": status,
            "log": log_path,
            "interfaces": interfaces,
            "shots": screen_path,
            'start_stamp': start,
            'start_time': start_time,
            'end_time': end_time,
            'end_stamp': end,
            'cost': cost
        }
        return case_data

    @staticmethod
    def remove_duplicate(json_contents):
        """去除失败重试的重复结果"""
        case_list = []
        no_repeat_tags = []
        for item in json_contents:
            full_name = item["full_name"]
            parameters = item["parameters"]
            if (full_name, parameters) not in no_repeat_tags:
                no_repeat_tags.append((full_name, parameters))
                case_list.append(item)
            else:
                for case in case_list:
                    if case['full_name'] == full_name and case['parameters'] == parameters:
                        if case['status'] != 'passed':
                            case_list.remove(case)
                            case_list.append(item)
        return case_list

    def get_result_list(self):
        """获取对外的测试结果列表"""
        file_list = self.get_files()
        result_list = []
        for file in file_list:
            content = self.get_file_content(file)
            parser_content = self.parser_content(content)
            result_list.append(parser_content)
        return self.remove_duplicate(result_list)

    def get_api_list(self):
        """获取接口列表"""
        tests = self.get_result_list()
        apis = []
        for test in tests:
            apis.extend(test["interfaces"])
        return list(set(apis))

    def get_statistical_data(self):
        case_list = self.get_result_list()

        # 获取用例统计数据
        passed_list = []
        fail_list = []
        broken_list = []
        skipped_list = []
        unknown_list = []
        for case in case_list:
            status = case.get('status')
            if status == 'passed':
                passed_list.append(case)
            elif status == 'failed':
                fail_list.append(case)
            elif status == 'broken':
                broken_list.append(case)
            elif status == 'skipped':
                skipped_list.append(case)
            else:
                unknown_list.append(case)
        total = len(case_list)
        passed = len(passed_list)
        failed = len(fail_list)
        broken = len(broken_list)
        skipped = len(skipped_list)
        unknown = len(unknown_list)
        rate = round((passed / total) * 100, 2)

        # 获取整个任务的开始和结束时间
        start_time_timestamp, end_time_timestamp = \
            case_list[0].get('start_stamp'), case_list[0].get('end_stamp')
        for case in case_list:
            inner_start = case.get('start_stamp')
            inner_end = case.get('end_stamp')
            if inner_start < start_time_timestamp:
                start_time_timestamp = inner_start
            if inner_end > end_time_timestamp:
                end_time_timestamp = inner_end

        # 时间戳转成日期
        start_time, end_time = time.strftime("%Y-%m-%d %H:%M:%S",
                                             time.localtime(start_time_timestamp / 1000)), \
                               time.strftime("%Y-%m-%d %H:%M:%S",
                                             time.localtime(end_time_timestamp / 1000))
        cost = (end_time_timestamp - start_time_timestamp) / 1000
        if cost > 60:
            cost = '{}min'.format(round(cost / 60, 1))
        else:
            cost = '{}s'.format(round((end_time_timestamp - start_time_timestamp) / 1000, 1))

        return {
            'total': total,
            'passed': passed,
            'failed': failed,
            'broken': broken,
            'skipped': skipped,
            'unknown': unknown,
            'rate': rate,
            'start': start_time,
            'end': end_time,
            'cost': cost,
            'tests': case_list
        }


def get_allure_data(result_path):
    """兼容邮件和钉钉的调用"""
    return AllureData(result_path).get_statistical_data()


if __name__ == '__main__':
    print(AllureData('../../demo/tests/report').get_statistical_data())



"""
pip install pandas==1.3.4
pip install openpyxl==3.0.9
pip install xlrd==2.0.1 --- 用于解析.xls文件
@Author: kang.yang
@Date: 2024/3/16 09:23
"""
import xlrd
import pandas as pd


class Excel(object):

    def __init__(self, file_name: str):
        self.file_name = file_name

    def read(self, sheet_name=0, row_index: int = None, col_index: int = None, col_name: str = None):
        if self.file_name.endswith(".xls"):
            workbook = xlrd.open_workbook(self.file_name, ignore_workbook_corruption=True)
            if col_index:
                df = pd.read_excel(workbook, sheet_name=sheet_name, usecols=[col_index-1])
                return [r[0] for r in df.values.tolist()]
            else:
                df = pd.read_excel(workbook, sheet_name=sheet_name)
        else:
            if col_index:
                df = pd.read_excel(self.file_name, sheet_name=sheet_name, usecols=[col_index-1])
                return [r[0] for r in df.values.tolist()]
            else:
                df = pd.read_excel(self.file_name, sheet_name=sheet_name)

        if col_name:
            return df[col_name].values.tolist()

        if row_index is not None:
            values = [df.values[row_index-1].tolist()]
        else:
            values = df.values.tolist()

        row_list = []
        col_names = []

        for name in df:
            col_names.append(name)

        for row in values:
            row_dict = {}
            for name, value in zip(col_names, row):
                row_dict[name] = value
            row_list.append(row_dict)

        return row_list

    def write(self, sheet_dict: dict, append=False):
        """
        :param sheet_dict:
        数据格式: {
            'sheet1_name': {'标题列1': ['张三', '李四'], '标题列2': [80, 90]},
            'sheet2_name': {'标题列3': ['王五', '郑六'], '标题列4': [100, 110]}
        }
        :param append: 是否追加，默认覆盖
        """
        df_dict = {}
        for sheet_name, sheet_data in sheet_dict.items():
            df_dict[sheet_name] = pd.DataFrame(sheet_data)

        writer = pd.ExcelWriter(self.file_name)
        for sheet_name, sheet_data in sheet_dict.items():
            _df = pd.DataFrame(sheet_data)
            if append:
                _df = df_dict[sheet_name].append(_df)
            _df.to_excel(writer, sheet_name=sheet_name, index=False)
        writer.save()


class CSV(object):

    def __init__(self, file_name):
        self.file_name = file_name

    def read(self, row_index: int = None, col_index: int = None, col_name: str = None):

        if col_index:
            df = pd.read_csv(self.file_name, usecols=[col_index - 1])
            res = [r[0] for r in df.values.tolist()]
            return res

        if col_name:
            df = pd.read_csv(self.file_name, usecols=[col_name])
            res = [r[0] for r in df.values.tolist()]
            return res

        df = pd.read_csv(self.file_name)
        if row_index:
            values = [df.values[row_index - 1].tolist()]
        else:
            values = df.values.tolist()

        row_list = []
        col_names = []

        for name in df:
            col_names.append(name)

        for row in values:
            row_dict = {}
            for name, value in zip(col_names, row):
                row_dict[name] = value
            row_list.append(row_dict)

        return row_list

    def write(self, data: dict, append=False):
        """
        :param data:
        数据格式：{
            '标题列1': ['张三', '李四'],
            '标题列2': [80, 90]
        }
        :param append: 是否追加，默认覆盖
        """
        df = pd.DataFrame(data)
        if append:
            _df = pd.read_csv(self.file_name)
            df = _df.append(df)
        df.to_csv(self.file_name, index=False)


if __name__ == '__main__':
    pass








"""
@Author: kang.yang
@Date: 2024/9/14 09:48
"""
from kytest.web import Elem, Page


class LoginPage(Page):
    url = "https://www-test.qizhidao.com/login?redirect=https%3A%2F%2Fwww-test.qizhidao.com%2F&businessSource" \
          "=PC%E7%BB%BC%E5%90%88-%E9%A1%B6%E9%83%A8%E6%A8%A1%E5%9D%97-%E7%AB%8B%E5%8D%B3%E7%99%BB%E5%BD%95&" \
          "registerPage=https%3A%2F%2Fwww-test.qizhidao.com%2F&fromPage=home"
    login_or_reg = Elem(text="登录/注册", exact=True)
    pwd_login = Elem(text='密码登录')
    phone_input = Elem(placeholder='请输入手机号码')
    pwd_input = Elem(placeholder='请输入密码')
    accept = \
        Elem(locator='form div') \
            .filter(has_text="我已阅读并同意《企知道平台服务协议》、 《企知道隐私权政策》、 《企知道商城/商贸空间使用须知》") \
            .locator("span") \
            .nth(3)
    login_now = Elem(role='button', name='立即登录')
    first_company = Elem().get_by_text('北京大胖涮锅城有限公司')



"""
@Author: kang.yang
@Date: 2023/11/16 17:50
"""
import kytest
from kytest.web import TestCase

from page.pub_page import PubPage


@kytest.story('登录模块')
class TestWebDemo(TestCase):
    def start(self):
        self.pub = PubPage(self.dr)

    @kytest.title("登录")
    def test_login(self):
        self.pub.login()
        self.assert_url()






from .elem import Elem
from .driver import Driver
from .case import TestCase
from .page import Page
from .util import IosUtil

__all__ = [
    "Elem",
    "Driver",
    "TestCase",
    "Page",
    "IosUtil"
]




"""
@Author: kang.yang
@Date: 2024/8/6 10:50
"""
import os
import sys
from . import __version__

login_content = '''import requests


def get_host_and_head(projectName: str, env: str = 'test'):
    """
    根据项目名获取登录态请求头
    :param env: 默认test
    :param projectName: qzd-bff-pcweb，目前只支持bff层的项目
    :return:
    """
    print("获取登录态数据")
    url = 'http://tmqa.qizhidao.net/api/common/getLoginHead'
    payload = {'projectName': projectName, "env": env}
    res = requests.get(url, params=payload)
    print(res.text)
    data = res.json()['data']
    host = data['host']
    head = data['headers']
    return host, head
'''


ignore_content = '''report
shot
.idea
.pytest_cache
__pycache__
debug.py
venv
generator.py
'''


api_run = '''import kytest
from data.login_data import get_host_and_head


if __name__ == '__main__':
    host, head = get_host_and_head('')
    kytest.main(
        host=host,
        headers=head,
        path='tests'
    )
'''

api_debug = '''import kytest
from data.login_data import get_host_and_head


if __name__ == '__main__':
    host, head = get_host_and_head('')
    kytest.main(
        host=host,
        headers=head,
        path='tests/test_demo.py'
    )
'''

api_generator = '''# 需要启动tms服务
from kytest.api import generate_case


if __name__ == '__main__':
    host = "http://localhost:8001"
    project_name = 'qzd-bff-patent'
    controller = 'Annualfee Message Controller'
    generate_case(host, project_name, controller)
'''

web_recorder = '''from kytest.web import record_case


if __name__ == '__main__':
    url = "https://www.qizhidao.com/login"
    record_case(url)
'''

app_run = '''import kytest


if __name__ == '__main__':
    kytest.main(
        did='',
        pkg='',
        path='tests'
    )
'''

app_debug = '''import kytest


if __name__ == '__main__':
    kytest.main(
        did='',
        pkg='',
        path='tests/test_demo.py'
    )
'''

web_run = '''import kytest


if __name__ == '__main__':
    kytest.main(
        host={'web': 'xxx'},
        headers={},
        path='tests'
    )
'''

web_debug = '''import kytest


if __name__ == '__main__':
    kytest.main(
        host={'web': 'xxx'},
        headers={},
        path='tests/test_web.py'
    )
'''


def create_scaffold(platform):
    """create scaffold with specified project name."""

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    # 新增测试数据目录
    create_folder("tests")
    create_folder("data")
    create_file('.gitignore', ignore_content)
    # 新增安卓测试用例
    if platform in ("android", "ios", "hm"):
        create_folder("page")
        create_file('run.py', app_run)
        create_file('debug.py', app_debug)
        create_file('requirements.txt', f'kytest[{platform}]=={__version__}')
    elif platform == "web":
        create_folder("page")
        create_file('run.py', web_run)
        create_file('debug.py', web_debug)
        create_file('recorder.py', web_recorder)
        create_file('requirements.txt', f'kytest[{platform}]=={__version__}')
    elif platform == "api":
        create_file('run.py', api_run)
        create_file('debug.py', api_debug)
        create_file('generator.py', api_generator)
        create_file(os.path.join("data", "login_data.py"), login_content)
        create_file('requirements.txt', f'kytest=={__version__}')
    else:
        print("请输入正确的平台: android、ios、hm、web、api")
        sys.exit()



"""
@Author: kang.yang
@Date: 2024/4/29 14:16
"""
from adbutils import adb


class AdrUtil:
    """
    单独拆开的原因是，有一些功能单独这个库就可以实现了，不用初始化
    https://github.com/openatx/adbutils
    """

    def __init__(self, device_id=None):
        if device_id is None:
            self.device = adb.device()
        else:
            self.device = adb.device(device_id)

    @staticmethod
    def get_connected():
        """
        获取设备列表
        @return:
        """
        device_list = adb.device_list()
        if device_list:
            return [device.serial for device in device_list]
        else:
            raise Exception("无已连接设备")

    @staticmethod
    def get_first_device():
        """
        获取第一个设备
        @return:
        """
        return AdrUtil.get_connected()[0]

    def shell(self, *args, **kwargs):
        """
        执行adb shell命令
        @param args:
        @param kwargs:
        @return: : ShellReturn(args='echo 1', returncode=0, output='1\n')
        """
        return self.device.shell2(*args, **kwargs)

    def click(self, x, y):
        """
        点击坐标
        @param x:
        @param y:
        @return:
        """
        self.device.click(x, y)

    def swipe(self, x1, y1, x2, y2):
        """
        从x1，y1滑动到x2，y2
        @param x1:
        @param y1:
        @param x2:
        @param y2:
        @return:
        """
        self.device.swipe(x1, y1, x2, y2)

    def input(self, text):
        """
        输入
        @param text:
        @return:
        """
        self.device.send_keys(text)

    def start_app(self, pkg, stop=True):
        """
        启动应用
        @param pkg:
        @param stop: 是否默认先关闭应用
        @return:
        """
        if stop is True:
            self.stop_app(pkg)
        self.device.app_start(pkg)

    def stop_app(self, pkg):
        """
        停止应用
        @param pkg:
        @return:
        """
        self.device.app_stop(pkg)

    def clear_app(self, pkg):
        """
        清空应用缓存
        @param pkg:
        @return:
        """
        self.device.app_clear(pkg)

    def install_app(self, apk, grant=True, nolaunch=False):
        """
        安装应用
        @param apk:
        @param grant：是否授予全部权限
        @param nolaunch：默认安装后直接启动应用
        @return:
        """
        flags = ["-r", "-t"]
        if grant is True:
            flags.append("-g")
        self.device.install(apk, flags=flags, nolaunch=nolaunch)

    def uninstall_app(self, pkg):
        """
        卸载应用
        @param pkg:
        @return:
        """
        self.device.uninstall(pkg)

    def running_apps(self):
        """
        运行中的应用列表
        @return:
        """
        return self.device.list_packages()

    def current_app(self):
        """
        当前应用
        @return:
        """
        return self.device.app_current()

    def push(self, *args, **kwargs):
        """
        推送内容到手机
        @param args:
        @param kwargs:
        @return:
        """
        self.device.sync.push(*args, **kwargs)

    def pull(self, *args, **kwargs):
        """
        从手机下载内容
        @param args:
        @param kwargs:
        @return:
        """
        self.device.sync.pull(*args, **kwargs)

    def app_info(self, pkg_name: str):
        """
        应用版本信息等
        @param pkg_name:
        @return:
        """
        return self.device.app_info(pkg_name)

    def device_info(self):
        """
        设备信息
        @return:
        """
        return self.device.info

    def window_size(self):
        return self.device.window_size()

    def press(self, key_code):
        """
        键盘点击
        @param key_code: HOME，详见：https://www.cnblogs.com/hujingnb/p/10282238.html
        @return:
        """
        self.device.keyevent(key_code)

    def dump(self):
        return self.device.dump_hierarchy()


if __name__ == '__main__':
    util = AdrUtil()










import os
import yaml

local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.yml')

    def __getitem__(self, key):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        try:
            return yaml_data['common'][key]
        except:
            return None

    def __setitem__(self, key, value):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        # if key in yaml_data['common']:
        #     yaml_data['common'][key] = value
        # else:
        #     raise KeyError(f'{key} not in {yaml_data["common"]}')
        yaml_data['common'][key] = value
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(yaml_data, f)

    def __len__(self):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        return len(yaml_data['common'])

    def __repr__(self):  # 用于终端显示，与本博文关系不大
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        return str(yaml_data['common'])

    def reset(self):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        yaml_data['common'] = {
            'base_url': None,
            'headers': None,
            'project': None,
            'web_url': None
        }
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(yaml_data, f)


kconfig = Config()

if __name__ == '__main__':
    print(kconfig['project'])




"""
@Author: kang.yang
@Date: 2024/9/24 09:11
"""
import subprocess
import time
import os


page_start = '''from kytest.web import Page, Elem


class RecordPage(Page):'''

case_start = '''import kytest

from page.record_page import RecordPage


@kytest.story('录制模块')
class TestRecord(kytest.WebTC):
    def start(self):
        self.rp = RecordPage(self.dr)

    @kytest.title("录制场景")
    def test_record(self):'''


def create_folder(path):
    os.makedirs(path)
    msg = f"created folder: {path}"
    print(msg)


def create_file(path, file_content=""):
    with open(path, "w", encoding="utf-8") as f:
        f.write(file_content)
    msg = f"created file: {path}"
    print(msg)


def record_case(url):
    """
    使用playwright codegen录制脚本并转换成page和test
    @param url:
    @return:
    """
    # 启动 playwright codegen 命令，保持非阻塞模式
    process = subprocess.Popen(['playwright',
                                'codegen',
                                url,
                                '-o',
                                'recorded.py'
                                ])

    while True:
        # 每秒钟检查一次进程状态
        return_code = process.poll()
        # return_code = 0

        # 打印 returncode，如果进程还未结束，returncode 会是 None
        print(f'Return code: {return_code}')

        if return_code == 0:
            with open('recorded.py', 'r') as f:
                scripts = f.readlines()
            page_content = page_start
            case_content = case_start
            for i, script in enumerate(scripts):
                script = script.strip()
                # 其它的特殊场景暂时先不管
                if script.startswith('page.') and 'page.close' not in script:
                    # 解析
                    if 'goto' in script:
                        url = script.split('"')[-2]
                        page_content += f'\n    url_{i} = "{url}"'
                        case_content += f'\n        self.rp.goto(self.rp.url_{i})'
                    else:
                        loc, operation = script.rsplit('.', 1)
                        loc = loc[4:]
                        elem_name = f'elem_{i}'
                        page_content += f'\n    {elem_name} = Elem(){loc}'
                        case_content += f'\n        self.rp.{elem_name}.{operation}'

            # 生成page：page/record_page.py => RecordPage
            # 如果没有page目录，新建一个
            if not os.path.exists('page'):
                create_folder('page')
            create_file(os.path.join('page', 'record_page.py'), page_content + '\n')
            # 生成test：tests/test_record.py => TestRecord
            if not os.path.exists('tests'):
                create_folder('tests')
            create_file(os.path.join('tests', 'test_record.py'), case_content + '\n')

            break

        # 等待 1 秒再进行下一次检查
        time.sleep(1)

    # 如果进程仍然在运行，在这里等待它完全结束
    process.wait()






"""
@Author: kang.yang
@Date: 2024/9/30 10:48
"""
import allure
from hmdriver2.driver import Driver
from hmdriver2.proto import DisplayRotation

from kytest.hm.util import HmUtil
from kytest.utils.log import logger
from kytest.utils.common import general_file_path


class HmDriver:
    """
    https://github.com/codematrixer/hmdriver2
    """

    def __init__(self, device_id: str = None):
        if device_id is None:
            self.d = Driver(HmUtil.get_first_device())
        else:
            self.d = Driver(device_id)

    # app管理
    def install_app(self, *args, **kwargs):
        logger.info("安装应用")
        self.d.install_app(*args, **kwargs)

    def uninstall_app(self, *args, **kwargs):
        logger.info("卸载应用")
        self.d.uninstall_app(*args, **kwargs)

    def list_apps(self):
        logger.info("获取应用列表")
        return self.d.list_apps()

    def start_app(self, pkg_name, page_name, force=True):
        """
        可以通过hdc命令获取hdc shell aa dump -l，如 ：
        (venv) iMac:kytest_project UI$ hdc shell aa dump -l
        User ID #100
          current mission lists:{
            Mission ID #98  mission name #[#com.qzd.hm:entry:EntryAbility]  lockedState #0  mission affinity #[]
              AbilityRecord ID #2043
                app name [com.qzd.hm]
                main name [EntryAbility]
                bundle name [com.qzd.hm]
                ability type [PAGE]
                state #FOREGROUND  start time [268869923]
                app state #FOREGROUND
                ready #1  window attached #0  launcher #0
                callee connections:
                isKeepAlive: false
         }


        @param force: 是否强制启动
        @param pkg_name: 包名，对应bundle name
        @param page_name: 页面名，对应main name
        @return:
        """
        logger.info("启动应用")
        if force is True:
            self.d.force_start_app(pkg_name, page_name)
        else:
            self.d.start_app(pkg_name, page_name)

    def stop_app(self, *args, **kwargs):
        logger.info("停止应用")
        self.d.stop_app(*args, **kwargs)

    def clear_app(self, *args, **kwargs):
        logger.info("清除app数据")
        self.d.clear_app(*args, **kwargs)

    def app_info(self, *args, **kwargs):
        logger.info("获取App详情")
        return self.d.get_app_info(*args, **kwargs)

    # 设备操作
    def device_info(self):
        logger.info("获取设备信息")
        return self.d.device_info

    def device_size(self):
        logger.info("获取设备分辨率")
        return self.d.display_size

    def rotation_status(self):
        logger.info("获取设备旋转状态")
        return self.d.display_rotation

    def set_rotation(self, status: int):
        """
        需要app支持旋转
        @param status:
        @return:
        """
        logger.info("设置设备旋转状态")
        if status == 0:
            self.d.set_display_rotation(DisplayRotation.ROTATION_0)
        elif status == 1:
            self.d.set_display_rotation(DisplayRotation.ROTATION_90)
        elif status == 2:
            self.d.set_display_rotation(DisplayRotation.ROTATION_180)
        elif status == 4:
            self.d.set_display_rotation(DisplayRotation.ROTATION_270)

    def home(self):
        logger.info("返回桌面首页")
        self.d.go_home()

    def back(self):
        logger.info("返回上一页")
        self.d.go_back()

    def screen_on(self):
        logger.info("点亮屏幕")
        self.d.screen_on()

    def screen_off(self):
        logger.info("息屏")
        self.d.screen_off()

    def unlock(self):
        logger.info("屏幕解锁")
        self.d.unlock()

    def press(self, key_code):
        """
        https://github.com/codematrixer/hmdriver2/blob/4d7bceaded947bd63d737de180064679ad4c77b8/hmdriver2/proto.py#L133
        @param key_code:
        @return:
        """
        logger.info("系统按键点击")
        self.d.press_key(key_code)

    def shell(self, *args, **kwargs):
        logger.info("执行hdc shell 命令")
        self.d.shell(*args, **kwargs)

    def open_url(self, url):
        logger.info("打开url或schema")
        self.d.open_url(url)

    def pull(self, *args, **kwargs):
        logger.info("从设备下载文件到本地")
        self.d.pull_file(*args, **kwargs)

    def push(self, *args, **kwargs):
        logger.info("将本地文件上传到设备")
        self.d.push_file(*args, **kwargs)

    def shot(self, file_name=None):
        """
        截图并上传allure
        @param file_name:
        @return:
        """
        file_path = general_file_path(file_name)
        logger.info(f"截图保存至: {file_path}")
        self.d.screenshot(file_path)

        logger.info("截图上传allure报告")
        allure.attach.file(
            file_path,
            attachment_type=allure.attachment_type.PNG,
            name=f"{file_path}",
        )
        return file_path

    def click(self, x, y):
        logger.info("点击操作")
        self.d.click(x, y)

    def double_click(self, x, y):
        logger.info("双击操作")
        self.d.double_click(x, y)

    def long_click(self, x, y):
        logger.info("长按操作")
        self.d.long_click(x, y)

    def swipe(self, *args, **kwargs):
        logger.info("滑动操作")
        self.d.swipe(*args, **kwargs)

    def input(self, *args, **kwargs):
        logger.info("输入操作")
        self.d.input_text(*args, **kwargs)

    def dump(self):
        logger.info("获取控件树")
        return self.d.dump_hierarchy()


if __name__ == '__main__':
    dr = HmDriver()
    dr.start_app('com.qzd.hm')





"""
@Author: kang.yang
@Date: 2024/10/10 11:09
"""
from typing import Literal


class BrowserConfig:
    browser_name: Literal[
                "chrome",
                "firefox",
                "webkit",
                "msedge"] = "chrome"
    headless: bool = False
    state: str = None
    maximized: bool = False
    window_size: list = None




from .element import Elem
from .driver import Driver
from .case import TestCase
from .page import Page
from .util import AdrUtil

__all__ = [
    "Elem",
    "Driver",
    "TestCase",
    "Page",
    "AdrUtil"
]



import logging
from loguru import logger


class PropogateHandler(logging.Handler):
    def emit(self, record) -> None:
        logging.getLogger(record.name).handle(record)


logger.add(PropogateHandler(), format="{time:YYYY-MM-DD at HH:mm:ss} | {message}")





"""
@Author: kang.yang
@Date: 2023/10/26 09:48
"""
import time
import allure

from .driver import HmDriver
from .element import Elem

from kytest.utils.log import logger
from kytest.utils.config import kconfig
from kytest.api.request import HttpReq
from kytest.running.conf import App


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    dr: HmDriver = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        try:
            cls().start_class()
        except BaseException as e:
            logger.error(f"start_class Exception: {e}")

    @classmethod
    def teardown_class(cls):
        try:
            cls().end_class()
        except BaseException as e:
            logger.error(f"end_class Exception: {e}")

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    # def config(self):
    #     """
    #     目前就是用来设置是否自动启动应用
    #     @return:
    #     """
    #     pass

    def setup_method(self):
        # # 设置调整
        # self.config()

        # 设置默认feature
        project_name = kconfig['project']
        if project_name:
            allure.dynamic.feature(project_name)

        # 驱动初始化
        self.dr = HmDriver(App.did)

        self.start()

    def teardown_method(self):
        self.end()

    # 公共方法
    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def shot(self, name: str, delay=0):
        """截图"""
        if delay:
            self.sleep(delay)
        self.dr.shot(name)

    # UI方法
    def elem(self, **kwargs):
        """
        元素定位方法
        @return:
        """
        return Elem(driver=self.dr, **kwargs)

    def uninstall_app(self):
        """
        卸载应用
        @return:
        """
        self.dr.uninstall_app(App.pkg)

    def install_app(self, apk_url):
        """
        安装应用
        @return:
        """
        self.dr.install_app(apk_url)

    def start_app(self, abilityName=None):
        """
        启动应用
        @return:
        """
        if abilityName is not None:
            self.dr.start_app(App.pkg, abilityName)
        else:
            if App.ability is not None:
                self.dr.start_app(App.pkg, App.ability)
            else:
                raise KeyError('ability不能为空')

    def stop_app(self):
        """
        停止应用
        @return:
        """
        self.dr.stop_app(App.pkg)

    def clear_app(self):
        """
        清空数据
        @return:
        """
        self.dr.clear_app(App.pkg)



import kytest
from kytest.adr import TestCase

from page.adr_page import AdrPage


@kytest.story('测试demo')
class TestAdrDemo(TestCase):
    def start(self):
        self.start_app()
        self.adr = AdrPage(self.dr)

    @kytest.title('进入设置页')
    def test_go_setting(self):
        if self.adr.adBtn.exists():
            self.adr.adBtn.click()
        self.adr.myTab.click()
        self.adr.setBtn.click()
        self.adr.page_title.assert_text_eq('设置')








"""
@Author: kang.yang
@Date: 2024/9/30 10:48
"""
from .element import Elem
from .driver import HmDriver as Driver
from .case import TestCase
from .page import Page

__all__ = [
    "Elem",
    "Driver",
    "TestCase",
    "Page"
]



from .driver import Driver
from .element import Elem
from .case import TestCase
from .page import Page
from .config import BrowserConfig
from .recorder import record_case

__all__ = [
    "Driver",
    "TestCase",
    "Elem",
    "Page",
    "BrowserConfig",
    "record_case"
]



"""
pip install nacos-sdk-python==0.1.14
@Author: kang.yang
@Date: 2024/5/7 16:50
"""
import nacos
import yaml


class NacosConfig:
    """
    nacos服务器配置
    """
    SERVER_ADDRESS = "http://nacos-idc-lan.qizhidao.net:8848"  # 服务器地址
    NAMESPACE = "f2afa4d4-6707-4170-a90c-f3f963075e15"  # 命名空间id
    USER_NAME = "qc_conf_user_dev"  # 账号
    PASSWORD = "2DYH1XfbS79FeM3y"  # 密码


class NacosApi:
    """
    nacos配置读写操作
    """

    def __init__(self,
                 host=NacosConfig.SERVER_ADDRESS,
                 namespace=NacosConfig.NAMESPACE,
                 user=NacosConfig.USER_NAME,
                 password=NacosConfig.PASSWORD):
        self.client = nacos.NacosClient(host,
                                        namespace=namespace,
                                        username=user,
                                        password=password)

    def get_by_data_id(self, data_id: str, key: str = None, group: str = 'DEFAULT_GROUP'):
        if not data_id:
            raise KeyError('data_id不能为空')

        nacos_config = self.client.get_config(data_id, group)
        dict_nacos_config = yaml.load(nacos_config, Loader=yaml.FullLoader)

        if key is not None:
            key_data = dict_nacos_config.get(key)
            print(key_data)
            return key_data
        else:
            print(dict_nacos_config)
            return dict_nacos_config


if __name__ == '__main__':
    pass





"""
@Author: kang.yang
@Date: 2024/8/27 17:01
"""
import base64


def encode(image_path: str):
    """
    图片内容加密成base64字符串
    @return:
    """
    # 读取图片内容
    with open(image_path, 'rb') as image_file:
        image_data = image_file.read()

    # 用base64进行加密
    encoded_image = base64.b64encode(image_data)  # 编码
    encoded_image_str = encoded_image.decode('utf-8')  # 转成字符串

    # 给加密串前面拼上前缀
    prefix = image_path.split('.')[-1]
    base64_image_with_prefix = f"data:image/{prefix};base64,{encoded_image_str}"

    return base64_image_with_prefix



"""
@Author: kang.yang
@Date: 2023/10/26 09:48
"""
import time
import allure

from .element import Elem
from .driver import Driver

from kytest.running.conf import App
from kytest.utils.log import logger
from kytest.utils.config import kconfig
from kytest.api.request import HttpReq


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    dr: Driver = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        try:
            cls().start_class()
        except BaseException as e:
            logger.error(f"start_class Exception: {e}")

    @classmethod
    def teardown_class(cls):
        try:
            cls().end_class()
        except BaseException as e:
            logger.error(f"end_class Exception: {e}")

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):

        # 设置默认feature
        project_name = kconfig['project']
        if project_name:
            allure.dynamic.feature(project_name)

        # 驱动初始化
        self.dr = Driver(App.did)

        self.start()

    def teardown_method(self):
        self.end()

    # 公共方法
    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def shot(self, name: str, delay=0):
        """截图"""
        if delay:
            self.sleep(delay)
        self.dr.shot(name)

    # UI方法
    def elem(self,
             rid=None,
             className=None,
             text=None,
             xpath=None,
             index=None):
        """
        元素定位方法
        @param rid:
        @param className:
        @param text:
        @param xpath:
        @param index:
        @return:
        """
        _kwargs = {}
        if rid is not None:
            _kwargs["resourceId"] = rid
        if className is not None:
            _kwargs["className"] = className
        if text is not None:
            _kwargs["text"] = text
        if xpath:
            _kwargs["xpath"] = xpath
        if index is not None:
            _kwargs["instance"] = index

        return Elem(driver=self.dr, **_kwargs)

    def uninstall_app(self):
        """
        卸载应用
        @return:
        """
        self.dr.util.uninstall_app(App.pkg)

    def install_app(self, apk_url):
        """
        安装应用
        @return:
        """
        self.dr.util.install_app(apk_url)

    def start_app(self):
        """
        启动应用
        @return:
        """
        self.dr.util.start_app(App.pkg)

    def stop_app(self):
        """
        停止应用
        @return:
        """
        self.dr.util.stop_app(App.pkg)

    def assert_activity(self, activity, timeout=10):
        self.dr.assert_activity(activity, timeout=timeout)




"""
@Author: kang.yang
@Date: 2024/4/29 09:31
"""
import os
import re
import shutil
import subprocess
import pytest
import json

from kytest.running.conf import App
from kytest.utils.log import logger
from kytest.utils.allure_util import AllureData


def _get_case_list(path: str):
    # 获取所有用例文件的完整路径列表
    _total_file_list = []
    for root, dirs, files in os.walk(path):
        if 'pycache' in root:
            continue
        files_str = ''.join(files)
        if '.py' not in files_str:
            continue
        files = [item for item in files if item != '__init__.py']
        if files:
            for _file in files:
                _total_file_list.append(os.path.join(root, _file))

    # 获取path目录下的用例原始字符串
    cases_str = subprocess.run(['pytest', path, '--collect-only'], capture_output=True, text=True).stdout

    # 把所有的标签拿出来
    lines = cases_str.split("\n")
    result = []

    for line in lines:
        match = re.search(r"<(.*?)>", line)
        if match:
            item = match.group(1)
            result.append(item)

    # 解析成用例列表
    case_list = []
    current_package = ''
    current_module = ''
    current_class = ''
    for item in result:
        if 'Package' in item:
            current_package = item.split(" ")[1]
        if 'Module' in item:
            current_module = item.split(" ")[1]
        if 'Class' in item:
            current_class = item.split(" ")[1]
        if 'Function' in item:
            _function = item.split(" ")[1].split("[")[0]
            _file_path = f"{current_package}/{current_module}"
            for item in _total_file_list:
                if _file_path in item:
                    _file_path = item
                    break
            print(f"{_file_path}::{current_class}::{_function}")
            case_list.append(f"{_file_path}::{current_class}::{_function}")

    # 去重
    print("去重后：")
    case_list = sorted(list(set(case_list)))
    for case in case_list:
        print(case)

    return case_list


def _move_file(source_dir, target_dir):
    # 获取源目录中的所有文件名列表
    file_list = os.listdir(source_dir)

    for file in file_list:
        # 构建源文件的完整路径
        source_file = os.path.join(source_dir, file)

        if not os.path.isdir(source_file):  # 判断是否为文件而非子目录
            # 构建目标文件的完整路径
            target_file = os.path.join(target_dir, file)

            try:
                # 移动文件至目标目录
                shutil.move(source_file, target_file)

                print('已移动文件：', file)
            except Exception as e:
                print('移动文件时发生错误：', str(e))


def _change_name_and_historyId(device_id, report_path):
    _allure = AllureData(result_path=report_path)
    result_list = _allure.get_files()
    for result in result_list:
        content = _allure.get_file_content(result)
        # 获取name，给name带上-{serial}的后缀
        content["name"] = content["name"] + "-" + device_id
        # 获取historyId，在后面带上-{serial}的后缀
        content["historyId"] = content["historyId"] + "-" + device_id
        # content重新写入回result中
        with open(result, "w") as f:
            f.write(json.dumps(content))


def _app_main(path, did, pkg, is_all=False):
    # 修改配置
    # App.serial = serial
    # App.package = package
    # App.udid = udid
    # App.bundle_id = bundle_id
    # App.ocr_service = ocr_api
    # App.auto_start = start
    App.did = did
    App.pkg = pkg

    # 执行用例
    # 由于是多设备并发执行，所以要对报告目录进行区分，后面增加合并的能力(有空调研一下pytest-xdist的报告合并实现方式)
    report_path = "report"
    if is_all is True:
        if did:
            report_path = f"report-{did}"
            cmd_list = [path, '-sv', '--alluredir', report_path]
            logger.info(cmd_list)
            pytest.main(cmd_list)
            # 进行合并，修改result.json中的name和historyId
            _change_name_and_historyId(did, report_path)

            # 把目录中的文件都移入report目录中
            if not os.path.exists('report'):
                os.makedirs('report')
            _move_file(report_path, 'report')

            # 删除原目录
            shutil.rmtree(report_path, ignore_errors=True)
        # if udid:
        #     report_path = f"report-{udid}"
        #     cmd_list = [path, '-sv', '--alluredir', report_path]
        #     logger.info(cmd_list)
        #     pytest.main(cmd_list)
        #     # 进行合并
        #     _change_name_and_historyId(udid, report_path)
        #
        #     # 把目录中的文件都移入report目录中
        #     if not os.path.exists('report'):
        #         os.makedirs('report')
        #     _move_file(report_path, 'report')
        #
        #     # 删除原目录
        #     shutil.rmtree(report_path, ignore_errors=True)
    else:
        cmd_list = [path, '-sv', '--alluredir', report_path]
        logger.info(cmd_list)
        pytest.main(cmd_list)



import time

import allure
import wda

from .util import IosUtil

from kytest.utils.log import logger
from kytest.utils.common import general_file_path


class Driver(object):
    """
    https://github.com/openatx/facebook-wda
    """

    def __init__(self, udid: str = None):
        if udid is None:
            self.device_id = IosUtil.get_first_device()
        else:
            self.device_id = udid
        logger.info(f"初始化ios驱动: {self.device_id}")

        self.port = int(self.device_id.split("-")[0][-4:])
        self.d = wda.USBClient(self.device_id,
                               port=self.port)
        self.util = IosUtil(self.device_id)

        if self.d.is_ready():
            logger.info('wda已就绪')
        else:
            logger.info('wda未就绪, 现在启动')
            self.util.start_wda()

    def back(self):
        logger.info("返回上一页")
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    def enter(self):
        logger.info("点击回车")
        self.d.send_keys("\n")

    def clear(self):
        logger.info("清空输入框")
        self.d.send_keys("")

    def input(self, text: str):
        logger.info(f"输入文本: {text}")
        self.d.send_keys(text)

    def click(self, x, y):
        logger.info(f"点击坐标: {x}, {y}")
        self.d.click(x, y)

    def swipe(self, x1, y1, x2, y2):
        """
        从x1，y1滑动到x2，y2

        @param x1:
        @param y1:
        @param x2:
        @param y2:
        @return:
        """
        logger.info(f"滑动")
        self.d.swipe(x1, y1, x2, y2)

    def shot(self, file_name=None):
        file_path = general_file_path(file_name)
        logger.info(f"截图保存至: {file_path}")
        self.d.screenshot(file_path)

        logger.info("截图上传allure报告")
        allure.attach.file(
            file_path,
            attachment_type=allure.attachment_type.PNG,
            name=f"{file_path}",
        )
        return file_path


if __name__ == '__main__':
    pass














import pytest
# import json
# import yaml


def order(index):
    """
    指定用例执行顺序, pip install pytest-ordering==0.6
    doc: https://blog.csdn.net/weixin_43880991/article/details/116221362
    """
    return pytest.mark.run(order=index)


def depend(depends: list or str = None, name=None):
    """
    设置用例依赖关系, pip install pytest-dependency==0.5.1
    doc: https://www.cnblogs.com/se7enjean/p/13513131.html
    """
    if isinstance(depends, str):
        depends = [depends]
    return pytest.mark.dependency(name=name, depends=depends)


# 参数化数据
def data(out_name, list_data: list):
    """
    必须传入一个list，使用时通过在参数列表传入parma进行调用
    """
    return pytest.mark.parametrize(out_name, list_data)


# # 从json文件获取数据进行参数化
# def file_data(file=None, key=None):
#     # logger.debug(config.get_env())
#
#     """
#     感觉excel和csv不常用，去掉
#     @param file: 文件名
#     @param key: 针对json和yaml文件
#     """
#     file_path = file  # 去掉文件查找机制，不好理解，用处也不大
#
#     # logger.debug(file_path)
#     if file_path.endswith(".json"):
#         content = read_json(file_path, key)
#     elif file_path.endswith(".yml"):
#         content = read_yaml(file_path, key)
#     # elif file_path.endswith(".csv"):
#     #     content = read_csv(file_path, row)
#     # elif file_path.endswith(".xlsx"):
#     #     content = read_excel(file_path, row)
#     else:
#         raise TypeError("不支持的文件类型，仅支持json、yml")
#
#     if content:
#         return data(content)
#     else:
#         raise ValueError('数据不能为空')


# def read_json(file_path, key=None):
#     """
#     读取json文件中的指定key
#     @return
#     """
#     with open(file_path, 'r', encoding='utf-8') as f:
#         json_data = json.load(f)
#     if isinstance(json_data, list):
#         return json_data
#     else:
#         if key:
#             return json_data[key]
#         raise ValueError('key不能为空')
#
#
# def read_yaml(file_path, key=None):
#     """
#     读取yaml文件中的指定key
#     @param file_path:
#     @param key:
#     """
#     with open(file_path, 'r', encoding='utf-8') as f:
#         yaml_data = yaml.load(f, Loader=yaml.FullLoader)
#     if isinstance(yaml_data, list):
#         return yaml_data
#     else:
#         if key:
#             return yaml_data[key]
#         raise ValueError('key不能为空')



"""
@Author: kang.yang
@Date: 2024/9/23 15:23
"""
from page.web_page import LoginPage
from data.user_data import USERNAME, PASSWORD


# 公共方法放在这个公共方法中
class PubPage:
    """登录模块公共方法"""

    def __init__(self, driver):
        self.lp = LoginPage(driver)

    def login(self, username=USERNAME, password=PASSWORD):
        """从首页进行登录"""
        self.lp.goto()
        self.lp.login_or_reg.click()
        self.lp.pwd_login.click()
        self.lp.phone_input.fill(username)
        self.lp.pwd_input.fill(password)
        self.lp.accept.click()
        self.lp.login_now.click()
        self.lp.first_company.click()




import allure
import uiautomator2 as u2

from kytest.utils.log import logger
from kytest.adr.util import AdrUtil
from kytest.utils.common import general_file_path


class Driver:
    """
    https://github.com/openatx/uiautomator2
    """

    def __init__(self, device_id=None):
        if device_id is None:
            device_id = AdrUtil.get_first_device()
        logger.info(f"初始化安卓驱动: {device_id}")

        self.d = u2.connect(device_id)
        self.util = AdrUtil(device_id)

    def shot(self, file_name=None):
        """
        比adb截图快，所以保留了
        @param file_name:
        @return:
        """
        file_path = general_file_path(file_name)
        logger.info(f"截图保存至: {file_path}")
        self.d.screenshot(file_path)

        logger.info("截图上传allure报告")
        allure.attach.file(
            file_path,
            attachment_type=allure.attachment_type.PNG,
            name=f"{file_path}",
        )
        return file_path

    def wait_app(self, pkg_name: str, front=False, timeout=20):
        """
        等待app运行
        @param pkg_name:
        @param front：应用保持在前台
        @param timeout：
        @return:
        """
        pid = self.d.app_wait(pkg_name, front=front, timeout=timeout)
        return pid

    def open_schema(self, schema: str):
        """
        # same as
        # adb shell am start -a android.intent.action.VIEW -d "appname://appnamehost"
        @param schema:
        @return:
        """
        self.d.open_url(schema)

    def session(self, *args, **kwargs):
        """
        Session represent an app lifecycle. Can be used to start app, detect app crash.

        # Launch and close app
        sess = d.session("com.netease.cloudmusic") # start 网易云音乐
        sess.close() # 停止网易云音乐
        sess.restart() # 冷启动网易云音乐

        # Use python with to launch and close app
        with d.session("com.netease.cloudmusic") as sess:
            sess(text="Play").click()

        # Attach to the running app
        # launch app if not running, skip launch if already running
        sess = d.session("com.netease.cloudmusic", attach=True)

        # Detect app crash
        # When app is still running
        sess(text="Music").click() # operation goes normal

        # If app crash or quit
        sess(text="Music").click() # raise SessionBrokenError
        # other function calls under session will raise SessionBrokenError too

        # check if session is ok.
        # Warning: function name may change in the future
        sess.running() # True or False
        @return:
        """
        return self.d.session(*args, **kwargs)

    def wait_activity(self, activity, timeout=10):
        """
        等待activity启动
        @param activity
        @param timeout
        @return: True or False
        """
        return self.d.wait_activity(activity, timeout)

    def assert_activity(self, activity, timeout=10):
        logger.info(f"断言页面activity: {activity}")
        assert self.wait_activity(activity, timeout=timeout)

    def device_info(self):
        return self.d.device_info

    def press(self, *args, **kwargs):
        self.d.press(*args, **kwargs)

    def unlock(self):
        self.d.unlock()

    def double_click(self, *args, **kwargs):
        self.d.double_click(*args, **kwargs)

    def long_click(self, *args, **kwargs):
        self.d.long_click(*args, **kwargs)

    def drag(self, *args, **kwargs):
        self.d.drag(*args, **kwargs)

    def dump(self, *args, **kwargs):
        return self.d.dump_hierarchy(*args, **kwargs)

    def input(self, *args, **kwargs):
        self.d.send_keys(*args, **kwargs)


if __name__ == '__main__':
    dr = Driver()

























import click
from . import __version__
from .scaffold import create_scaffold


@click.group()
@click.version_option(version=__version__, help="Show version.")
# 老是变，等最后定下来再搞，目前也没啥用
def main():
    pass


@main.command()
@click.option('-p', '--platform', help="Specify the platform.")
def create(platform):
    """
    创建新项目
    @param platform: 平台，如api、android、ios、web
    @return:
    """
    create_scaffold(platform)


@main.command()
@click.option('-p', '--platform', help="Specify the platform.")
@click.option('-u', '--url', help="Specify the url.")
def inspector(platform, url):
    """
    获取元素定位信息
    @param platform: 平台，android、ios、hm、web
    @param url：针对web，需要录制的页面的url
    @return:
    """
    import os
    if platform in ['android', 'ios', 'hm', 'web']:
        os.system('uiviewer')
    elif platform == 'web':
        if not url:
            raise KeyError('url不能为空')
        os.system(f'playwright codegen {url}')
    else:
        raise KeyError('只支持app、web')




# pip install yagmail==0.15.293
import traceback
import yagmail

from kytest.utils.allure_util import get_allure_data
from kytest.utils.log import logger
from kytest.testdata import get_now_datetime


class Mail:
    def __init__(self, host, port, user, password):
        self.host = host
        self.port = port
        self.username = user
        self.password = password

    def send_email(self, receivers, subject, contents, attachments=None):
        """
        发送邮件
        @param receivers: 收件人列表
        @param subject: 邮件主题
        @param contents: 邮件内容
        @param attachments: 附件文件的绝对路径
        @return:
        """
        try:
            logger.info(f'开始发送邮件~')
            # 初始化服务对象直接根据参数给定，更多参考SMTP(）内部
            server = yagmail.SMTP(host=self.host, port=self.port,
                                  user=self.username, password=self.password)
            # 发送内容，设置接受人等信息，更多参考SMTP.send()内部
            server.send(to=receivers,
                        subject=subject,
                        contents=contents,
                        attachments=attachments)
            server.close()
        except Exception:
            logger.info(f'发送失败~')
            print('traceback.format_exc(): {}'.format(traceback.format_exc()))
            return False

        # 无任何异常表示发送成功
        logger.info(f'发送成功~')
        return True

    def send_report(self, to_list, title, report_url):
        allure_data = get_allure_data('report')
        total = allure_data.get('total')
        passed = allure_data.get('passed')
        fail = total - passed
        rate = allure_data.get('rate')

        # 邮件内容
        title = f'{title}({get_now_datetime(strftime=True)})'
        body_str = '\n\n共 {0} 个用例，通过 {1} 个，失败 {2} 个，通过率 {3}%，详见: {4}'\
            .format(total, passed, fail, rate, report_url)

        # 发送邮件
        if fail > 0:
            self.send_email(to_list, title, body_str)
        else:
            logger.info('用例全部成功，无需通知~')


if __name__ == '__main__':
    pass








# @Time    : 2022/2/22 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : request.py
import json as json_util
import logging
import requests
import jmespath

from functools import wraps
from requests.packages import urllib3
from urllib import parse
from jsonschema import validate, ValidationError

from kytest.utils.log import logger
from kytest.utils.config import kconfig

# 去掉requests本身的日志
urllib3_logger = logging.getLogger("urllib3")
urllib3_logger.setLevel(logging.CRITICAL)

# 去掉不设置证书的报警
urllib3.disable_warnings()


def formatting(msg):
    """formatted message"""
    if isinstance(msg, dict):
        return json_util.dumps(msg, indent=2, ensure_ascii=False)
    return msg


def request(func):

    @wraps(func)
    def wrapper(*args, **kwargs):
        logger.info("-------------- Request -----------------[🚀]")
        # 给接口带上默认域名
        # 从配置文件中读取域名
        host = kconfig['base_url']
        # 如果接口路径不以http开头，把域名写到key为url的位置参数中或者第一个参数中
        if "url" in kwargs:
            path: str = kwargs.get("url", "")
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                kwargs["url"] = url
            else:
                url = path
        else:
            path = list(args)[1]
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                args_list = list(args)
                args_list[1] = url
                args = tuple(args_list)
            else:
                url = path

        # 请求头处理，写入登录态
        default_headers: dict = kconfig['headers']
        user_set_headers = kwargs.pop("headers", {})
        _final_headers = {}
        if default_headers:
            _final_headers.update(default_headers)
        if user_set_headers:
            _final_headers.update(user_set_headers)
        if _final_headers:
            kwargs["headers"] = _final_headers

        # 更新超时时间
        timeout_user_set = kwargs.pop("timeout", None)  # 用例脚本中设置的超时时间
        kwargs["timeout"] = timeout_user_set if timeout_user_set else 10

        # 发送请求
        r = func(*args, **kwargs)

        # 输出请求参数日志
        logger.debug("[method]: {m}      [url]: {u}".format(m=func.__name__.upper(), u=url))
        auth = kwargs.get("auth", "")
        if auth:
            logger.debug(f"[auth]:\n {formatting(auth)}")
        logger.debug(f"[headers]:\n {formatting(dict(r.request.headers))}")
        cookies = kwargs.get("cookies", "")
        if cookies:
            logger.debug(f"[cookies]:\n {formatting(cookies)}")
        params = kwargs.get("params", "")
        if params:
            logger.debug(f"[params]:\n {formatting(params)}")
        data = kwargs.get("static", "")
        if data:
            logger.debug(f"[static]:\n {formatting(data)}")
        json = kwargs.get("json", "")
        if json:
            logger.debug(f"[json]:\n {formatting(json)}")

        # 保存响应结果并输出日志
        status_code = r.status_code
        headers = r.headers
        content_type = headers.get("Content-Type")
        ResponseResult.status_code = status_code
        logger.info("-------------- Response ----------------")
        logger.debug(f"[status]: {status_code}")
        logger.debug(f"[headers]: {formatting(headers)}")
        try:
            resp = r.json()
            logger.debug(f"[type]: json")
            logger.debug(f"[response]:\n {formatting(resp)}")
            ResponseResult.response = resp
        except Exception:
            # 非json响应数据，根据响应内容类型进行判断
            logger.info("response is not json type static.")
            if content_type is not None:
                if "text" not in content_type:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.content}")
                    ResponseResult.response = r.content
                else:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.text}")
                    ResponseResult.response = r.text
            else:
                logger.debug('ContentType为空，响应异常！！！')
                ResponseResult.response = r.text

        return r

    return wrapper


class ResponseResult:
    # 并发执行不会串数据，是因为我用的是多进程而不是多线程吧???
    status_code = 200
    response = None


class HttpReq(object):
    @request
    def get(self, url, params=None, verify=False, **kwargs):
        return requests.get(url, params=params, verify=verify, **kwargs)

    @request
    def post(self, url, data=None, json=None, verify=False, **kwargs):
        return requests.post(url, data=data, json=json, verify=verify, **kwargs)

    @request
    def put(self, url, data=None, json=None, verify=False, **kwargs):
        if json is not None:
            data = json_util.dumps(json)
        return requests.put(url, data=data, verify=verify, **kwargs)

    @request
    def delete(self, url, verify=False, **kwargs):
        return requests.delete(url, verify=verify, **kwargs)

    @property
    def response(self):
        """
        Returns the result of the response
        :return: response
        """
        return ResponseResult.response

    # 断言
    @staticmethod
    def assertStatus(status_code):
        """
        状态码
        """
        actual_code = ResponseResult.status_code
        logger.info(f"断言: {actual_code} 等于 {status_code}")
        assert (actual_code == status_code), f"状态码错误: {actual_code}"

    @staticmethod
    def assertSchema(schema, response=None) -> None:
        """
        Assert JSON Schema
        doc: https://json-schema.org/
        """
        logger.info(f"assertSchema -> {formatting(schema)}.")

        if response is None:
            response = ResponseResult.response

        try:
            validate(instance=response, schema=schema)
        except ValidationError as msg:
            assert "Response static" == "Schema static", msg

    @staticmethod
    def assertEq(path: str, value):
        """
        等于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 等于 {value}")
                    assert _search_value == value, f'匹配到的第{i + 1}个结果: {_search_value} 不等于 {value}'
            else:
                logger.info(f"断言: {search_value} 等于 {value}")
                assert search_value == value, f"{search_value} 不等于 {value}"
        else:
            logger.info(f"断言: {search_value} 等于 {value}")
            assert search_value == value, f"{search_value} 不等于 {value}"

    @staticmethod
    def assertNotEq(path: str, value):
        """
        不等于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 不等于 {value}")
                    assert _search_value != value, f'匹配到的第{i + 1}个结果: {_search_value} 不应该等于 {value}'
            else:
                logger.info(f"断言: {search_value} 不等于 {value}")
                assert search_value != value, f'{search_value} 不应该等于 {value}'
        else:
            logger.info(f"断言: {search_value} 不等于 {value}")
            assert search_value != value, f'{search_value} 不应该等于 {value}'

    @staticmethod
    def assertLenEq(path: str, value):
        """
        长度等于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if search_value:
            for _search_value in search_value:
                if not isinstance(_search_value, list):
                    logger.info(f"断言: {len(search_value)} 等于 {value}")
                    assert len(search_value) == value, f'{len(search_value)} 不等于 {value}'
                    break
                else:
                    logger.info(f"断言: {len(_search_value)} 等于 {value}")
                    assert len(_search_value) == value, f'{len(_search_value)} 不等于 {value}'
        else:
            logger.info(f"断言: {len(search_value)} 等于 {value}")
            assert len(search_value) == value, f'{len(search_value)} 不等于 {value}'

    @staticmethod
    def assertLenGt(path: str, value):
        """
        长度大于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if search_value:
            for _search_value in search_value:
                if not isinstance(_search_value, list):
                    logger.info(f"断言: {len(search_value)} 大于 {value}")
                    assert len(search_value) > value, f'{len(search_value)} 应该大于 {value}'
                    break
                else:
                    logger.info(f"断言: {len(_search_value)} 大于 {value}")
                    assert len(_search_value) > value, f'{len(_search_value)} 应该大于 {value}'
        else:
            logger.info(f"断言: {len(search_value)} 大于 {value}")
            assert len(search_value) > value, f'{len(search_value)} 应该大于 {value}'

    @staticmethod
    def assertLenLt(path: str, value):
        """
        长度小于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if search_value:
            for _search_value in search_value:
                if not isinstance(_search_value, list):
                    logger.info(f"断言: {len(search_value)} 小于 {value}")
                    assert len(search_value) < value, f'{len(search_value)} 应该小于 {value}'
                    break
                else:
                    logger.info(f"断言: {len(_search_value)} 小于 {value}")
                    assert len(_search_value) < value, f'{len(_search_value)} 应该小于 {value}'
        else:
            logger.info(f"断言: {len(search_value)} 小于 {value}")
            assert len(search_value) < value, f'{len(search_value)} 应该小于 {value}'

    @staticmethod
    def assertGt(path: str, value):
        """
        大于
        """

        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 大于 {value}")
                    assert float(_search_value) > value, f'匹配到的第{i + 1}个结果: {_search_value} 应该大于 {value}'
            else:
                assert False, '匹配结果为空，断言失败'
        else:
            logger.info(f"断言: {search_value} 大于 {value}")
            assert float(search_value) > value, f"{search_value} 应该大于 {value}"

    @staticmethod
    def assertLt(path: str, value):
        """
        小于
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 小于 {value}")
                    assert float(_search_value) < value, f'匹配到的第{i + 1}个结果: {_search_value} 应该小于 {value}'
            else:
                assert False, '匹配结果为空，断言失败'
        else:
            logger.info(f"断言: {search_value} 小于 {value}")
            assert float(search_value) < value, f"{search_value} 应该小于 {value}"

    @staticmethod
    def assertRg(path: str, start, end):
        """
        范围
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 大于等于{start}，小于等于{end}")
                    assert (float(_search_value) >= start) and (float(_search_value) <= end), \
                        f'匹配到的第{i + 1}个结果: {_search_value} 不在[{start}, {end}]范围内'
            else:
                assert False, '匹配结果为空，断言失败'
        else:
            logger.info(f"断言: {search_value} 大于等于{start}，小于等于{end}")
            assert (float(search_value) >= start) and (float(search_value) <= end), \
                f'{search_value} 不在[{start}, {end}]范围内'

    @staticmethod
    def assertIn(path: str, value):
        """
        被包含
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 被 {value} 包含")
                    assert _search_value in value, f'匹配到的第{i + 1}个结果: {_search_value} 应该被 {value} 包含'
            else:
                logger.info(f"断言: {search_value} 被 {value} 包含")
                assert search_value in value, f"{search_value} 应该被 {value} 包含"
        else:
            logger.info(f"断言: {search_value} 被 {value} 包含")
            assert search_value in value, f"{search_value} 应该被 {value} 包含"

    @staticmethod
    def assertNotIn(path: str, value):
        """
        不被包含
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 不被 {value} 包含")
                    assert _search_value not in value, f'匹配到的第{i + 1}个结果: {_search_value} 应该不被 {value} 包含'
            else:
                logger.info(f"断言: {search_value} 不被 {value} 包含")
                assert search_value not in value, f"{search_value} 应该不被 {value} 包含"
        else:
            logger.info(f"断言: {search_value} 不被 {value} 包含")
            assert search_value not in value, f"{search_value} 应该不被 {value} 包含"

    @staticmethod
    def assertCt(path: str, value):
        """
        包含
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 包含 {value}")
                    assert value in _search_value, f'匹配到的第{i + 1}个结果: {_search_value} 应该包含 {value}'
            else:
                logger.info(f"断言: {search_value} 包含 {value} ")
                assert value in search_value, f"{search_value} 应该包含 {value}"
        else:
            logger.info(f"断言: {search_value} 包含 {value} ")
            assert value in search_value, f"{search_value} 应该包含 {value}"

    @staticmethod
    def assertNotCt(path: str, value):
        """
        包含
        """
        logger.info(f'查找"{path}"')
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, list):
            if search_value:
                for i, _search_value in enumerate(search_value):
                    logger.info(f"断言: {_search_value} 不包含 {value}")
                    assert value not in _search_value, f'匹配到的第{i + 1}个结果: {_search_value} 不应该包含 {value}'
            else:
                logger.info(f"断言: {search_value} 不包含 {value} ")
                assert value not in search_value, f"{search_value} 不应该包含 {value}"
        else:
            logger.info(f"断言: {search_value} 不包含 {value} ")
            assert value not in search_value, f"{search_value} 不应该包含 {value}"

    # @staticmethod
    # def assertType(path: str, value_type):
    #     """
    #     字段类型
    #     """
    #     if not isinstance(value_type, type):
    #         if value_type == "int":
    #             value_type = int
    #         elif value_type == "str":
    #             value_type = str
    #         elif value_type == "list":
    #             value_type = list
    #         elif value_type == "dict":
    #             value_type = dict
    #         else:
    #             value_type = str
    #
    #     search_value = jmespath.search(path, ResponseResult.response)
    #     logger.info(f"断言: {search_value} 是 {value_type} 类型")
    #     assert isinstance(
    #         search_value, value_type
    #     ), f"{search_value} 不是 {value_type} 类型"
    #
    # @staticmethod
    # def assertStart(path: str, value):
    #     """
    #     以什么开始
    #     """
    #     search_value: str = jmespath.search(path, ResponseResult.response)
    #     logger.info(f"断言: {search_value} 以 {value} 开头")
    #     assert search_value.startswith(value), f"{search_value} 不以 {value} 开头"
    #
    # @staticmethod
    # def assertEnd(path: str, value):
    #     """
    #     以什么结束
    #     """
    #     search_value: str = jmespath.search(path, ResponseResult.response)
    #     logger.info(f"断言: {search_value} 以 {value} 结尾")
    #     assert search_value.endswith(value), f"{search_value} 不以 {value} 结尾"
    #
    # @staticmethod
    # def assertRegex(path: str, value):
    #     """正则匹配"""
    #     search_value = jmespath.search(path, ResponseResult.response)
    #     match_obj = re.match(r"" + value, search_value, flags=re.I)
    #     logger.info(f"断言: {search_value} 匹配正则表达式 {value} 成功")
    #     assert match_obj is not None, f"结果 {search_value} 匹配失败"



"""
@Author: kang.yang
@Date: 2023/8/1 18:21
"""
import cv2
import time
import allure

from kytest.utils.log import logger


def draw_red_by_rect(image_path: str, rect: tuple):
    """在图片上画框，范围是左上角坐标和宽高"""
    # 读取图像
    image = cv2.imread(image_path)

    # 定义标记范围的坐标
    x, y, w, h = rect

    # 在图像上绘制矩形
    cv2.rectangle(image, (x, y), (x + w, y + h), (0, 0, 255), 2)

    # 保存标记后的图像
    cv2.imwrite(image_path, image)


def draw_red_by_coordinate(image_path: str, rect: tuple):
    """在图片上画框，范围是左上角坐标和右上角坐标"""
    """x_top_left, y_top_left, x_bottom_right, y_bottom_right"""
    # 读取图像
    image = cv2.imread(image_path)

    # 定义标记范围的坐标
    x, y, x1, y1 = rect

    # 在图像上绘制矩形
    cv2.rectangle(image, (x, y), (x1, y1), (0, 0, 255), 2)

    # 保存标记后的图像
    cv2.imwrite(image_path, image)


def cut_half(image, position):
    """把图片分成上下两半"""
    if position == "up":
        return image[:image.shape[0] // 2, :]
    elif position == "down":
        return image[image.shape[0] // 2:, :]
    else:
        raise KeyError("position传值错误")


def cut_by_position(image_path: str, position: str):
    """把图片按左上、左下、右上、右下进行分割"""
    logger.info(position)
    # 读取图像
    logger.info("分割图片")
    start = time.time()
    image = cv2.imread(image_path)
    # 获取图像的宽度和高度
    height, width, _ = image.shape
    logger.debug(f'{height}, {width}')
    # 计算每个切割区域的宽度和高度
    sub_width = width // 2
    sub_height = height // 2
    # 切割图像成上下左右四个等份
    if position == 'TL':
        image_content = image[0:sub_height, 0:sub_width]
    elif position == 'TR':
        image_content = image[0:sub_height, sub_width:width]
    elif position == 'BL':
        image_content = image[sub_height:height, 0:sub_width]
    elif position == 'BR':
        image_content = image[sub_height:height, sub_width:width]
    else:
        raise KeyError(f"position传值错误 all: {position}")

    new_path = f"{image_path.split('.')[0]}_{position}.{image_path.split('.')[1]}"
    logger.debug(new_path)
    cv2.imwrite(new_path, image_content)
    cut_height, cut_width, _ = image_content.shape
    logger.debug(f'{cut_height, cut_width}')
    end = time.time()
    logger.info(f"分割成功， 耗时: {end - start}s")
    info = {
        "path": new_path,
        "height": height,
        "width": width,
        "cut_height": cut_height
    }
    logger.debug(info)
    return info


def cut_and_upload(file_path, position: str = None):
    # 对图片进行分割
    info = None
    if position is not None:
        info = cut_by_position(file_path, position)
        file_path = info.get("path")
    # 上传allure报告
    allure.attach.file(
        file_path,
        attachment_type=allure.attachment_type.PNG,
        name=f"{file_path}",
    )
    if position is not None:
        return info
    else:
        return file_path







import typing

from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from kytest.utils.log import logger
from kytest.adr.driver import Driver


# 链式调用临时存储定位方式
class ChildLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'child'


class SiblingLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'sibling'


class LeftLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'left'


class RightLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'right'


class UpLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'up'


class DownLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'down'


class Elem(object):
    """
    安卓控件定义
    https://github.com/openatx/uiautomator2
    """

    def __init__(self, driver: Driver = None, watch: list = None, **kwargs):
        """
        @param driver: 安卓驱动
        @param watch: 需要处理的异常弹窗定位方式列表
        """
        self._kwargs = kwargs
        self._driver = driver
        self._xpath = kwargs.get('xpath', None)
        self._watch = watch
        self.locators = []

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # 链式调用方法
    def child(self, *args, **kwargs):
        self.locators.append(ChildLocator(*args, **kwargs))
        return self

    def sibling(self, *args, **kwargs):
        self.locators.append(SiblingLocator(*args, **kwargs))
        return self

    def left(self, *args, **kwargs):
        self.locators.append(LeftLocator(*args, **kwargs))
        return self

    def right(self, *args, **kwargs):
        self.locators.append(RightLocator(*args, **kwargs))
        return self

    def up(self, *args, **kwargs):
        self.locators.append(UpLocator(*args, **kwargs))
        return self

    def down(self, *args, **kwargs):
        self.locators.append(DownLocator(*args, **kwargs))
        return self

    # 公共方法
    def watch_handler(self):
        """
        异常弹窗处理
        @return:
        """
        logger.info(f"开始弹窗检测: {self._watch}")
        ctx = self._driver.d.watch_context()
        for loc in self._watch:
            ctx.when(loc).click()
        ctx.wait_stable()
        ctx.close()
        logger.info("检测结束")

    def find(self, timeout=5, n=3):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @param n：失败后重试的次数
        @return:
        """
        logger.info(f"查找: {self._kwargs}")
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath is not None else self._driver.d(**self._kwargs)

        # 链式调用方法叠加
        if self.locators:
            logger.info(f"链式调用: {self.locators}")
            for loc_obj in self.locators:
                if loc_obj.type == "child":
                    _element = _element.child(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'sibling':
                    _element = _element.sibling(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'left':
                    _element = _element.left(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'right':
                    _element = _element.right(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'up':
                    _element = _element.up(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'down':
                    _element = _element.down(*loc_obj.args, **loc_obj.kwargs)

        if self._watch:
            self.watch_handler()

        retry_count = n
        if _element.wait(timeout=timeout):
            logger.info(f"查找成功")
            return _element
        else:
            if retry_count > 0:
                for count in range(1, retry_count + 1):
                    logger.info(f"第{count}次重试...")
                    if self._watch:
                        self.watch_handler()
                    if _element.wait(timeout=timeout):
                        logger.info(f"查找成功")
                        return _element

            logger.info("查找失败")
            self._driver.shot("查找失败")
            raise Exception(f"控件: {self._kwargs}, 查找失败")

    # 属性获取
    def get_text(self, timeout=5):
        logger.info("获取文本")
        _elem = self.find(timeout=timeout)
        if isinstance(_elem, XPathSelector):
            elems = _elem.all()
        else:
            elems = list(_elem)
        text = []
        for elem in elems:
            text.append(elem.get_text())

        if len(text) == 1:
            text = text[0]

        logger.info(f"获取成功: {text}")
        return text

    def exists(self, timeout=5):
        logger.info("是否存在")
        result = False
        try:
            _element = self.find(timeout=timeout, n=0)
            result = True
        except:
            result = False
        finally:
            logger.info(result)
            return result

    def count(self, timeout=5):
        logger.info("获取定位到的控件数量")
        count = self.find(timeout=timeout).count
        logger.info(count)
        return count

    def info(self, timeout=5):
        logger.info("获取控件信息")
        info = self.find(timeout=timeout).info
        logger.info(info)
        return info

    def center(self, timeout=5, *args, **kwargs):
        return self.find(timeout=timeout).center(*args, **kwargs)

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector],
                      offset=(0.5, 0.5)):
        """
        修正控件中心坐标
        """
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    # 操作
    def click(self, timeout=5):
        logger.info("点击")
        element = self.find(timeout=timeout)
        x, y = self._adapt_center(element)
        self._driver.util.click(x, y)
        logger.info("点击完成")

    def long_click(self, timeout=5):
        logger.info("长按")
        element = self.find(timeout=timeout)
        x, y = self._adapt_center(element)
        self._driver.long_click(x, y)
        logger.info("长按完成")

    def input(self, text, timeout=5, pwd_check=False):
        logger.info(f"输入: {text}")
        self.click(timeout=timeout)
        if pwd_check is True:
            self._driver.d(focused=True).set_text(text)
        else:
            self._driver.util.input(text)
        logger.info("输入完成")

    def clear_text(self, timeout=5, *args, **kwargs):
        logger.info("清空")
        self.find(timeout=timeout).clear_text(*args, **kwargs)
        logger.info("清空完成")

    def screenshot(self, file_path, timeout=5):
        logger.info("截屏")
        self.find(timeout=timeout).screenshot().save(file_path)
        logger.info("截屏完成")

    def drag_to(self, timeout=5, *args, **kwargs):
        logger.info("拖动")
        self.find(timeout=timeout).drag_to(*args, **kwargs)
        logger.info("拖动完成")

    def assert_exists(self, timeout=5):
        logger.info("断言控件存在")
        try:
            assert self.exists(timeout=timeout)
        except Exception as e:
            raise e
        finally:
            logger.info("断言完成")

    def assert_text_eq(self, text, timeout=5):
        logger.info(f"断言控件文本等于: {text}")
        assert text == self.get_text(timeout=timeout)

    def assert_text_ct(self, text, timeout=5):
        logger.info(f"断言控件文本包括: {text}")
        assert text in self.get_text(timeout=timeout)


if __name__ == '__main__':
    pass










"""
@Author: kang.yang
@Date: 2023/10/26 09:48
"""
import time
import allure
import requests

from kytest.api.request import HttpReq
from kytest.utils.log import logger
from kytest.utils.config import kconfig
from kytest.utils.allure_util import AllureData


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """
    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        project_name = kconfig['project']
        if project_name:
            allure.dynamic.feature(project_name)

        self.start()

    def teardown_method(self):
        self.end()

    # 公共方法
    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    @staticmethod
    def check_coverage_data(backend_url, project_name):
        """
        检查接口平台已覆盖的接口在脚本中是否都已覆盖
        @param backend_url: tms平台后端服务域名，如: http://localhost:8001
        @param project_name: tms平台已导入接口项目名，如：kz-bff-patent
        @return:
        """
        apis_covered = AllureData().get_api_list()
        url = f"{backend_url}/api/api_test/api/check_test_result"
        body = {
            "project_name": project_name,
            "apis_covered": apis_covered
        }
        res = requests.post(url, json=body)

        if res.json()['data']['status'] == 'pass':
            print("都已真实覆盖。")
        else:
            miss_list = res.json()['data']['error_list']
            print(f"还有{len(miss_list)}个接口漏掉了，如下：")
            for api in miss_list:
                print(api)




"""
@Author: kang.yang
@Date: 2024/9/30 10:53
"""
import os
from kytest.utils.log import logger


class HmUtil:

    @staticmethod
    def get_connected():
        """
        获取设备列表
        @return:
        """
        cmd = 'hdc list targets'
        output = os.popen(cmd).read()
        print(output)
        device_list = [item.split(' ')[0] for item in output.split('\n') if item]
        if len(device_list) > 0:
            logger.info(f"已连接设备列表: {device_list}")
            return device_list
        else:
            raise Exception("无已连接设备")

    @staticmethod
    def get_first_device():
        """
        获取第一个设备
        @return:
        """
        return HmUtil.get_connected()[0]


if __name__ == '__main__':
    print(HmUtil.get_first_device())



"""
@Author: kang.yang
@Date: 2023/5/12 20:49
"""
import allure
import os

from typing import Literal
from playwright.sync_api import sync_playwright, expect

from kytest.utils.log import logger
from kytest.utils.common import general_file_path


class Driver:
    """
    https://playwright.dev/python/docs/locators
    """

    def __init__(
            self,
            browserName: Literal[
                "chrome",
                "firefox",
                "webkit",
                "msedge"] = "chrome",
            headless: bool = False,
            state: str = None,
            maximized: bool = False,
            window_size: list = None
    ):
        """
        浏览器驱动
        @param browserName: 浏览器类型，默认chrome，还支持firefox和webkit
        @param headless: 是否使用无头模式
        @param state: 使用state.json加载登录态
        @param maximized: 是否使用全屏模式
        @param window_size: 指定窗口分辨率，如[1920, 1080]
        """
        # browName判断
        support_browser_list = [
            "chromium",
            "chrome",
            "chrome-beta",
            "msedge",
            "msdege-beta",
            "medege-dev",
            "firefox",
            "webkit"
        ]
        if browserName not in support_browser_list:
            raise KeyError(f"不支持的浏览器类型: {browserName}")

        logger.info("初始化playwright驱动")
        logger.info(f"安装浏览器")
        os.system(f"playwright install")
        if headless is True and window_size is None:
            window_size = [1920, 1080]

        self.playwright = sync_playwright().start()
        _kwargs = {"headless": headless}
        if maximized and window_size is None:
            _kwargs["args"] = ['--start-maximized']
        if browserName == 'firefox':
            self.browser = self.playwright.firefox.launch(**_kwargs)
        elif browserName == 'webkit':
            self.browser = self.playwright.webkit.launch(**_kwargs)
        elif 'msedge' in browserName:
            _kwargs["channel"] = 'msedge'
            self.browser = self.playwright.chromium.launch(**_kwargs)
        else:
            self.browser = self.playwright.chromium.launch(**_kwargs)

        _context_kwargs = {"storage_state": state}
        if maximized and window_size is None:
            _context_kwargs["no_viewport"] = True
        if window_size:
            _context_kwargs["viewport"] = {'width': window_size[0], 'height': window_size[1]}
        self.context = self.browser.new_context(**_context_kwargs)
        self.page = self.context.new_page()

    def dialog_handler(self, _type: str = "dismiss"):
        """
        监听dialog，默认取消或者同意
        @param _type:
        @return:
        """
        if _type == "dismiss":
            self.page.on("dialog", lambda dialog: dialog.dismiss())
        else:
            self.page.on("dialog", lambda dialog: dialog.accept())

    def goto(self, url):
        logger.info(f"访问页面: {url}")
        self.page.goto(url)

    def storage_state(self, path=None):
        logger.info("保存浏览器状态信息")
        if not path:
            raise ValueError("路径不能为空")
        self.context.storage_state(path=path)

    @property
    def page_content(self):
        """获取页面内容"""
        logger.info("获取页面内容")
        content = self.page.content()
        logger.info(content)
        return content

    def add_cookies(self, cookies: list):
        logger.info("添加cookie并刷新页面")
        self.context.add_cookies(cookies)
        self.page.reload()

    def shot(self, file_name=None):
        file_path = general_file_path(file_name)
        logger.info(f"保存至: {file_path}")
        self.page.screenshot(path=file_path)
        logger.info("截图上传allure报告")
        allure.attach.file(
            file_path,
            attachment_type=allure.attachment_type.PNG,
            name=f"{file_path}",
        )
        return file_path

    def press(self, key):
        logger.info("键盘点击")
        self.page.keyboard.press(key)

    def back(self):
        logger.info("返回上一页")
        self.page.go_back()

    def close(self):
        logger.info("关闭浏览器")
        self.page.close()
        self.context.close()
        self.browser.close()
        self.playwright.stop()

    def assert_title(self, title: str, timeout: int = 5):
        logger.info(f"断言页面标题等于: {title}")
        expect(self.page).to_have_title(title,
                                        timeout=timeout * 1000)

    def assert_url(self, url: str, timeout: int = 5):
        logger.info(f"断言页面url等于: {url}")
        expect(self.page).to_have_url(url,
                                      timeout=timeout * 1000)


if __name__ == '__main__':
    pass



"""
@Author: kang.yang
@Date: 2024/8/24 09:24
"""
import sys
import os
import requests
from urllib.parse import urljoin


def generate_case(host: str, project: str, controller: str, base_path: str = 'tests'):
    """
    生成用例
    @param host: tms服务的域名
    @param project: tms服务导入的项目
    @param controller: 指定的controller，不指定则生成全部
    @param base_path：指定生成的目录，默认当前目录
    @return:
    """

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    if not host:
        print('host不能为空')
        sys.exit()
    if not project:
        print('project不能为空')
        sys.exit()

    url = urljoin(host, '/api/api_test/api/get_apis_by_project_and_controller')
    query = {
        "project_name": project,
        "controller": controller
    }
    try:
        res = requests.get(url, params=query).json()
    except requests.exceptions.ConnectionError:
        print(f"{host}服务连接异常~")
        sys.exit()
    if res["code"] == 0:
        api_list = res["data"]
        # 生成用例
        for api in api_list:
            controller = api["controller"]
            if base_path is not None:
                controller_path = os.path.join(base_path, controller)
            else:
                controller_path = controller
            if not os.path.exists(controller_path):
                create_folder(controller_path)
            test_name = api["test_name"]
            print(f"生成用例: {test_name}")
            test_script = api["test_script"]
            create_file(os.path.join(controller_path, test_name), test_script)
        print(f"共生成{len(api_list)}条用例.")
    else:
        print("用例生成异常")
        print(res)
        sys.exit()



"""
@Author: kang.yang
@Date: 2023/11/16 17:52
"""
import kytest

from requests_toolbelt import MultipartEncoder


@kytest.story('pc首页')
class TestApiDemo(kytest.TC):

    @kytest.title('get/post请求')
    def test_normal_req(self):
        url = '/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        params = {"type": 2}
        self.post(url, json=params)
        self.assertEq('data[*].showType', 2)

    # @kytest.title("form请求")
    # def test_form_req(self):
    #     url = '/qzd-bff-patent/image-search/images'
    #     m = MultipartEncoder(
    #         fields={
    #             # 'key1': 'value1',
    #             'imageFile': ('logo.png', open('data/logo.png', 'rb'), 'image/png')
    #         }
    #     )
    #     self.post(url, data=m, headers={'Content-Type': m.content_type})
    #     self.assertEq("code", 0)



"""
@Author: kang.yang
@Date: 2024/4/27 14:39
"""
import time
import os
import subprocess
import shutil

from kytest.utils.log import logger


class IosUtil:
    """
    tidevice常用功能的封装，单独拆开就不用初始化fk了
    https://github.com/alibaba/tidevice
    """

    def __init__(self, device_id=None):
        if device_id is None:
            self.device_id = IosUtil.get_first_device()
        else:
            self.device_id = device_id

    @staticmethod
    def get_all_device():
        """获取当前连接的设备列表"""
        cmd = 'tidevice list'
        output = os.popen(cmd).read()
        device_list = [item.split(' ')[0] for item in output.split('\n') if item]
        if len(device_list) > 0:
            logger.info(f"已连接设备列表: {device_list}")
            return device_list
        else:
            raise Exception("无已连接设备")

    @staticmethod
    def get_first_device():
        """获取当前连接的第一个设备"""
        return IosUtil.get_all_device()[0]

    def uninstall_app(self, pkg_name):
        """卸载应用"""
        logger.info("卸载应用")
        cmd = f"tidevice -u {self.device_id} uninstall {pkg_name}"
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{self.device_id} 卸载应用{pkg_name} 成功")
            return
        else:
            logger.info(f"{self.device_id} 卸载应用{pkg_name}失败，因为{output}")

    def install_app(self, ipa_url):
        """
        安装应用
        """
        logger.info("安装应用")
        cmd = f"tidevice -u {self.device_id} install {ipa_url}"
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{self.device_id} 安装应用{ipa_url} 成功")
            return
        else:
            logger.info(f"{self.device_id} 安装应用{ipa_url}失败，因为{output}")

    def start_wda(self, wda_bundle_id=None):
        """
        启动wda
        @param wda_bundle_id:
        @return:
        """
        xctool_path = shutil.which("tidevice")
        args = []
        if self.device_id is not None:
            args.extend(["-u", self.device_id])
        args.append("wdaproxy")
        port = int(self.device_id.split("-")[0][-4:])
        args.extend(["--port", str(port)])
        if wda_bundle_id is not None:
            args.extend(["-B", wda_bundle_id])
        p = subprocess.Popen([xctool_path] + args)
        time.sleep(3)
        if p.poll() is not None:
            raise Exception("wda启动失败，可能是手机未连接")

    def start_app(self, pkg_name, stop=True):
        """启动应用"""
        logger.info("启动应用")
        if stop is True:
            self.stop_app(pkg_name)
        cmd = f"tidevice -u {self.device_id} launch {pkg_name}"
        output = subprocess.getoutput(cmd)
        print(output)

    def stop_app(self, pkg_name):
        """杀掉应用"""
        logger.info("杀掉应用")
        cmd = f"tidevice -u {self.device_id} kill {pkg_name}"
        output = subprocess.getoutput(cmd)
        print(output)

    @staticmethod
    def app_list():
        logger.info("查看应用列表")
        cmd = 'tidevice applist'
        output = os.popen(cmd).read()
        app_list = [item.split(' ')[0] for item in output.split('\n') if item]
        return app_list

    @staticmethod
    def running_app_list():
        logger.info("查看运行中应用列表")
        cmd = 'tidevice ps --json'
        output = os.popen(cmd).read()
        import json
        json_obj = json.loads(output)
        app_list = [item['bundle_id'] for item in json_obj if item]
        return app_list


if __name__ == '__main__':
    util = IosUtil()
    print(util.running_app_list())










"""
pip install pycryptodome==3.14.1
@Author: kang.yang
@Date: 2024/3/18 14:46
"""
import base64
from Crypto.Cipher import AES


def encode(key, src_str):
    """
    AES加密，一种对称加密算法
    @param key: 事先定好的秘钥
    @param src_str: 需要加密的字符串
    @return:
    """
    aes = AES.new(str.encode(key), AES.MODE_ECB)
    encode_pwd = str.encode(src_str.rjust(16, '@'))
    encrypt_str = str(base64.encodebytes(aes.encrypt(encode_pwd)), encoding='utf-8')
    return encrypt_str


def decode(key, ciphertext):
    """
    AES解密，一种对称加密算法
    @param key: 事先定好的秘钥
    @param ciphertext: 需要解密的密文
    @return:
    """
    aes = AES.new(str.encode(key), AES.MODE_ECB)
    decrypt_str = (aes.decrypt(base64.decodebytes(ciphertext.encode(encoding='utf-8'))).
                   decode().replace('@', ''))
    return decrypt_str





"""
@Author: kang.yang
@Date: 2023/5/13 10:16
"""
from playwright.sync_api import expect, Locator

from .driver import Driver

from kytest.utils.log import logger


# 链式调用临时存储定位方式
class RoleLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'role'


class TextLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'text'


class LabelLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'label'


class PlaceholderLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'placeholder'


class AltTextLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'alt_text'


class TitleLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'title'


class TestIdLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'test_id'


class FrameLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'frame'


class LocatorLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'locator'


class FilterLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'filter'


class NthLocator:

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.type = 'nth'


class FirstLocator:

    def __init__(self):
        self.type = 'first'


class LastLocator:

    def __init__(self):
        self.type = 'last'


class Elem:
    """
    通过playwright定位的web元素
    https://playwright.dev/python/docs/locators
    """

    def __init__(self,
                 driver: Driver = None,
                 role=None,
                 name=None,
                 text=None,
                 label=None,
                 placeholder=None,
                 alt_text=None,
                 title=None,
                 test_id=None,
                 frame_loc=None,
                 locator=None,
                 exact=False,
                 watch=False
                 ):

        self._driver = driver
        self._kwargs = {}

        self.role = role
        if self.role:
            self._kwargs['role'] = role
        self.name = name
        if self.name:
            self._kwargs['name'] = name
        self.text = text
        if self.text:
            self._kwargs['text'] = text
        self.label = label
        if self.label:
            self._kwargs['label'] = label
        self.placeholder = placeholder
        if self.placeholder:
            self._kwargs['placeholder'] = placeholder
        self.alt_text = alt_text
        if self.alt_text:
            self._kwargs['alt_text'] = alt_text
        self.title = title
        if self.title:
            self._kwargs['title'] = title
        self.test_id = test_id
        if self.test_id:
            self._kwargs['test_id'] = test_id
        self.frame_loc = frame_loc
        if self.frame_loc:
            self._kwargs['frame_loc'] = frame_loc
        self.loc = locator
        if self.loc:
            self._kwargs['locator'] = locator
        self.exact = exact
        if self.exact:
            self._kwargs['exact'] = exact
        self.locators = []
        self._watch = watch

    def __call__(self, *args, **kwargs):
        return self

    def __get__(self, instance, owner):
        """pm模式的关键"""
        if instance is None:
            return None
        self._driver = instance.driver
        return self

    # 链式调用方法
    def get_by_role(self, *args, **kwargs):
        self.locators.append(RoleLocator(*args, **kwargs))
        return self

    def get_by_text(self, *args, **kwargs):
        self.locators.append(TextLocator(*args, **kwargs))
        return self

    def get_by_label(self, *args, **kwargs):
        self.locators.append(LabelLocator(*args, **kwargs))
        return self

    def get_by_placeholder(self, *args, **kwargs):
        self.locators.append(PlaceholderLocator(*args, **kwargs))
        return self

    def get_by_alt_text(self, *args, **kwargs):
        self.locators.append(AltTextLocator(*args, **kwargs))
        return self

    def get_by_title(self, *args, **kwargs):
        self.locators.append(TitleLocator(*args, **kwargs))
        return self

    def get_by_test_id(self, *args, **kwargs):
        self.locators.append(TestIdLocator(*args, **kwargs))
        return self

    def frame_locator(self, *args, **kwargs):
        self.locators.append(FrameLocator(*args, **kwargs))
        return self

    def locator(self, *args, **kwargs):
        self.locators.append(LocatorLocator(*args, **kwargs))
        return self

    def filter(self, *args, **kwargs):
        self.locators.append(FilterLocator(*args, **kwargs))
        return self

    def nth(self, *args, **kwargs):
        self.locators.append(NthLocator(*args, **kwargs))
        return self

    @property
    def first(self):
        self.locators.append(FirstLocator())
        return self

    @property
    def last(self):
        self.locators.append(LastLocator())
        return self

    def get_first_locator(self):
        logger.info(f"第一定位: {self._kwargs}")
        if self.role is not None:
            if self.name is not None:
                return self._driver.page.get_by_role(self.role, name=self.name)
            else:
                return self._driver.page.get_by_role(self.role)
        elif self.text is not None:
            if self.exact is True:
                return self._driver.page.get_by_text(self.text, exact=True)
            else:
                return self._driver.page.get_by_text(self.text)
        elif self.label is not None:
            return self._driver.page.get_by_label(self.label)
        elif self.placeholder is not None:
            return self._driver.page.get_by_placeholder(self.placeholder)
        elif self.alt_text is not None:
            return self._driver.page.get_by_alt_text(self.alt_text)
        elif self.title is not None:
            return self._driver.page.get_by_title(self.title)
        elif self.test_id is not None:
            return self._driver.page.get_by_test_id(self.test_id)
        elif self.frame_loc is not None:
            return self._driver.page.frame_locator(self.frame_loc)
        elif self.loc is not None:
            return self._driver.page.locator(self.loc)
        else:
            logger.info("第一个元素为空")
            return self._driver.page

    def find(self, timeout=5, n=3):
        """查找指定的一个元素"""
        logger.info(f"查找元素")
        element = self.get_first_locator()
        # 链式调用方法叠加
        if self.locators:
            logger.info(f"链式调用: {self.locators}")
            for loc_obj in self.locators:
                if loc_obj.type == "role":
                    element = element.get_by_role(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'text':
                    element = element.get_by_text(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'label':
                    element = element.get_by_label(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'placeholder':
                    element = element.get_by_placeholder(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'alt_text':
                    element = element.get_by_alt_text(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'title':
                    element = element.get_by_title(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'test_id':
                    element = element.get_by_test_id(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'frame':
                    element = element.frame_locator(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'locator':
                    element = element.locator(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'filter':
                    element = element.filter(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'nth':
                    element = element.nth(*loc_obj.args, **loc_obj.kwargs)
                elif loc_obj.type == 'first':
                    element = element.first
                elif loc_obj.type == 'last':
                    element = element.last

        try:
            element.wait_for(timeout=timeout * 1000)
            return element
        except:
            retry_count = n
            if retry_count > 0:
                for count in range(1, retry_count + 1):
                    logger.info(f"第{count}次重试...")
                    if self._watch:
                        self._driver.dialog_handler()
                    try:
                        element.wait_for(timeout=timeout * 1000)
                        # if self._debug is True:
                        #     element.evaluate('(element) => element.style.border = "2px solid red"')
                        #     time.sleep(1)
                        #     self._driver.shot("查找成功")
                        return element
                    except:
                        continue

            self._driver.shot("查找失败")
            raise Exception("查找失败")

    # 属性
    def is_visible(self, timeout=1):
        logger.info("是否可见")
        return self.find(timeout=timeout, n=0).is_visible()

    def is_hidden(self, timeout=1):
        logger.info("是否隐藏")
        return self.find(timeout=timeout, n=0).is_hidden()

    def text_content(self):
        logger.info("获取文本")
        elem = self.find()
        text = elem.text_content()
        logger.info(text)
        return text

    def all_text_content(self):
        logger.info("获取多个文本")
        elems = self.find().all()
        text_list = [elem.text_content() for elem in elems]
        logger.info(text_list)
        return text_list

    # 操作
    def click(self, timeout=5, *args, **kwargs):
        """
        page.get_by_text("Item").click(button="right")
        page.get_by_text("Item").click(modifiers=["Shift"])
        page.get_by_text("Item").click(position={ "x": 0, "y": 0})
        page.get_by_role("button").click(force=True)
        @param timeout: 
        @param args: 
        @param kwargs: 
        @return: 
        """
        logger.info("点击")
        self.find(timeout=timeout).click(*args, **kwargs)

    def dispatch_event_click(self, timeout=5):
        """
        其它点击方式都不管用了可以试试这个
        @param timeout:
        @return:
        """
        logger.info("js事件点击")
        self.find(timeout=timeout).dispatch_event('click')

    def dbclick(self, timeout=5):
        logger.info("双击")
        self.find(timeout=timeout).dblclick()

    def fill(self, text, timeout=5):
        logger.info(f"输入")
        self.find(timeout=timeout).fill(text)

    def press(self, key, timeout=5):
        """
        @param key: 单个键位支持Backquote, Minus, Equal, Backslash, Backspace, Tab, Delete, Escape,
        ArrowDown, End, Enter, Home, Insert, PageDown, PageUp, ArrowRight, ArrowUp, F1 - F12, Digit0 - Digit9,
        KeyA - KeyZ, etc.
                    还支持组合键，如Shift+A、Control+o、Control+Shift+T
        @param timeout:
        @return:
        """
        logger.info(f"键盘点击")
        self.find(timeout=timeout).press(key)

    def press_sequentially(self, words: str, timeout=5):
        """
        没找到引用，不知道靠不靠谱
        @param words:
        @param timeout:
        @return:
        """
        logger.info(f"逐个输入")
        self.find(timeout=timeout).press_sequentially(words)

    def check(self, timeout=5):
        logger.info("复选框选中")
        self.find(timeout=timeout).check()

    def select_option(self, value: str, timeout=5):
        logger.info("下拉选择")
        self.find(timeout=timeout).select_option(value)

    def set_input_file(self, timeout=5, *args, **kwargs):
        """
        @param timeout:
        @param args:
        支持单个文件：'file.pdf'
        支持多个文件：['file1.txt', 'file2.txt']
        清空：[]
        @param kwargs: Upload buffer from memory
        files=[
            {"name": "test.txt", "mimeType": "text/plain", "buffer": b"this is a test"}
        ]
        @return:
        """
        logger.info("输入框型文件上传")
        self.find(timeout=timeout).set_input_files(*args, **kwargs)

    def set_files(self, file_path: str, timeout=5):
        logger.info("动态生成型文件上传")
        locator = self.find(timeout=timeout)
        with self._driver.page.expect_file_chooser() as fc_info:
            locator.click()
        file_chooser = fc_info.value
        file_chooser.set_files(file_path)

    def focus(self, timeout=5):
        logger.info("聚焦")
        self.find(timeout=timeout).focus()

    def drag_to(self, locator: Locator, timeout=5):
        logger.info("拖动到另外一个元素")
        self.find(timeout=timeout).drag_to(locator)

    def drag_to_manually(self, locator: Locator, timeout=5):
        logger.info("手动模拟拖动")
        self.find(timeout=timeout).hover()
        self._driver.page.mouse.down()
        locator.hover()
        self._driver.page.mouse.up()

    def scroll_into_view_if_needed(self, timeout=5):
        """
        正常做其他操作会自动滚动到可视区，自动不生效再用这个方法
        @param timeout:
        @return:
        """
        logger.info("元素滚动到可视区")
        self.find(timeout=timeout).scroll_into_view_if_needed()

    def download(self, save_path: str, timeout=5):
        """
        下载
        @param save_path: 只要目录，文件名会自己生成
        @param timeout:
        @return:
        """
        logger.info("下载")
        with self._driver.page.expect_download() as download_info:
            self.find(timeout=timeout).click()
        download = download_info.value
        download.save_as(save_path + download.suggested_filename)

    def popup(self, timeout=5):
        """
        点击打开新页签，并返回新页签的page
        @param timeout:
        @return:
        """
        logger.info("打开新页签")
        with self._driver.page.expect_popup() as popup:
            self.find(timeout=timeout).click()
        new_page = popup.value
        return new_page

    # 断言
    def assert_checked(self, timeout=5):
        logger.info("断言：复选框是否选中")
        expect(self.find(timeout=timeout)).to_be_checked()

    def assert_visible(self, timeout=5):
        logger.info("断言：是否可见")
        expect(self.find(timeout=timeout)).to_be_visible()

    def assert_hidden(self, timeout=5):
        logger.info("断言：是否隐藏")
        expect(self.find(timeout=timeout)).to_be_hidden()

    def assert_text_ct(self, text: str, timeout=5):
        logger.info(f"断言：是否包含文本-{text}")
        expect(self.find(timeout=timeout)).to_contain_text(text)

    def assert_text_eq(self, text: str, timeout=5):
        logger.info(f"断言：是否等于文本-{text}")
        expect(self.find(timeout=timeout)).to_have_text(text)

    def assert_count_eq(self, count, timeout=5):
        logger.info(f"断言：locator定位到几个元素")
        expect(self.find(timeout=timeout)).to_have_count(count)


if __name__ == '__main__':
    pass



"""
@Author: kang.yang
@Date: 2023/8/1 18:21
"""
import random
import socket
import time
import os


def general_file_path(file_name):
    if not file_name:
        raise ValueError("文件名不能为空")

    # 截图并保存到当前目录的image文件夹中
    relative_path = "shots"
    # 把文件名处理成test.png的样式
    if "." in file_name:
        file_name = file_name.split(r".")[0]
    if os.path.exists(relative_path) is False:
        os.mkdir(relative_path)

    time_str = time.strftime(f"%Y%m%d%H%M%S")
    file_name = f"{time_str}_{file_name}.jpg"
    file_path = os.path.join(relative_path, file_name)
    return file_path


def is_port_in_use(port: int) -> bool:
    """判断端口是否已占用"""
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('127.0.0.1', port)) == 0


def get_free_port():
    """获取空闲端口"""
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('127.0.0.1', 0))
        try:
            return s.getsockname()[1]
        finally:
            s.close()
    except OSError:
        # bind 0 will fail on Manjaro, fallback to random port
        # https://github.com/openatx/adbutils/issues/85
        for _ in range(20):
            port = random.randint(10000, 20000)
            if not is_port_in_use(port):
                return port
        raise RuntimeError("No free port found")





# @Time    : 2022/2/11 9:00
# @Author  : kang.yang@qizhidao.com
# @File    : swagger.py
import requests
import urllib3

urllib3.disable_warnings()


def is_chinese(string):
    """
    检查整个字符串是否包含中文
    :param string: 需要检查的字符串
    :return: bool
    """
    for ch in string:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True

    return False


def get_swagger_data(swagger_url):
    """
    通过swagger接口获取接口列表
    @param swagger_url:
    @return: [
        ['请求方法', '项目名', '模块名', '模块描述', '接口名', '接口描述'],
        ...
    ]
    """
    # 请求url，获取返回的json
    res = requests.get(swagger_url, verify=False)
    # print(res.text)
    data_json: dict = res.json()
    # print(data_json)
    # 获取接口所属模块
    project: str = data_json.get('basePath')
    project = project.split('/')[1]
    # 获取tag名称和描述的映射关系
    tags = data_json.get('tags')
    tag_dict = {}
    for tag in tags:
        name = tag.get('name')
        des = tag.get('description')
        if name not in tag_dict:
            tag_dict[name] = des
    # print(tag_dict)
    # 获取接口信息
    paths = data_json.get('paths')
    print(len(paths))
    api_list = []
    for apiPath, value in paths.items():
        apiPath = f'/{project}{apiPath}'
        # apiPath = apiPath.replace('{', '')
        # apiPath = apiPath.replace('}', '')
        for method, content in value.items():
            # print(content)
            tags = content['tags']
            moduleName = ""
            moduleDesc = ""
            for tag in tags:
                if is_chinese(tag):
                    moduleDesc = tag
                    try:
                        moduleName = tag_dict[tag]
                        break
                    except:
                        continue
                else:
                    moduleName = tag
                    try:
                        moduleDesc = tag_dict[tag]
                        break
                    except:
                        continue

            moduleName = moduleName.strip()
            moduleDesc = moduleDesc.replace("'", "").replace('"', '').strip()
            # moduleDesc = moduleDesc.replace('"', '')

            apiDesc: str = content['summary']
            apiDesc = apiDesc.replace("'", "")
            apiDesc = apiDesc.replace('"', '')
            params = [item for item in content.get('parameters', [])]
            for i, param in enumerate(params):
                _in = param.get("in")
                _param = None
                if _in == 'body':
                    _schema = param.get("schema")
                    if _schema.get("originalRef", None) is not None:
                        _ref = _schema["originalRef"]
                        try:
                            _param = data_json["definitions"][_ref]['properties']
                        except:
                            _param = param
                            _param["type"] = "--"
                    else:
                        _items = _schema.get("items", None)
                        if _items is not None:
                            if _items.get('originalRef', None) is not None:
                                _ref = _schema["items"]["originalRef"]
                                _param = data_json["definitions"][_ref]['properties']
                                _param = [_param]
                            else:
                                _param = param
                                _param["type"] = param["schema"]["type"]
                                _param = [_param]
                        else:
                            _param = param
                            _param["type"] = param["schema"]["type"]

                if _param is not None:
                    params[i] = {"in": "body", "value": _param}

            # 把参数进行分类
            header_params = []
            path_params = []
            query_params = []
            form_params = []
            body_params = []
            other_params = []
            for param in params:
                try:
                    param_type = param['in']
                    if param_type == 'header':
                        header_params.append(param)
                    elif param_type == 'path':
                        path_params.append(param)
                    elif param_type == 'query':
                        query_params.append(param)
                    elif param_type == 'body':
                        value = param["value"]
                        if isinstance(value, list):
                            for item in value:
                                if 'in' in item:
                                    body_params.append({
                                        "name": item["name"],
                                        "in": "body",
                                        "description": item["description"],
                                        "type": item["type"]
                                    })
                                else:
                                    for _key, _value in value[0].items():
                                        body_params.append({
                                            "name": _key,
                                            "in": "body",
                                            "description": _value["description"],
                                            "type": _value["description"]
                                        })
                        else:
                            if 'in' in value:
                                body_params.append({
                                    "name": value['name'],
                                    'in': 'body',
                                    'description': value['description'],
                                    'type': value['type']
                                })
                            else:
                                for key, _value in value.items():
                                    body_params.append({
                                        "name": key,
                                        'in': 'body',
                                        'description': _value.get('description', "--"),
                                        'type': _value.get('type', "--")
                                    })
                    elif param_type == "formData":
                        form_params.append(param)
                    else:
                        other_params.append(param)
                except Exception as e:
                    print(str(e))
                    # print(param)
                    other_params.append(param)

            api_list.append({
                'projectName': project,
                'controller': moduleName,
                'controller_desc': moduleDesc,
                'method': method,
                'apiPath': apiPath,
                'apiDesc': apiDesc,
                'header_params': header_params,
                'path_params': path_params,
                'query_params': query_params,
                'form_params': form_params,
                'body_params': body_params,
                'other_params': other_params
            })
    for index, api in enumerate(api_list):
        print(index, api)
    return api_list


if __name__ == '__main__':
    api_list = get_swagger_data("http://app-test-lan.qizhidao.com/qzd-bff-pcweb/v2/api-docs?group=%E4%BC%81%E7%9F%A5%E9%81%93")



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from kytest import __version__, __description__

try:
    long_description = open(os.path.join('kytest', "README.md"), encoding='utf-8').read()
except IOError:
    long_description = ""

setup(
    name="kytest",
    version=__version__,
    description=__description__,
    author="杨康",
    author_email="772840356@qq.com",
    url="https://gitee.com/bluepang2021/ktest_project",
    platforms="API/Android/IOS/HM/WEB",
    packages=find_packages(),
    long_description=long_description,
    python_requires='>=3.8',
    classifiers=[
        "Programming Language :: Python :: 3.8"
    ],
    include_package_data=True,
    package_data={
        r'': ['*.yml'],
    },
    install_requires=[
        'requests==2.31.0',
        'requests-toolbelt==1.0.0',
        'urllib3==1.26.15',
        'jmespath==0.9.5',
        'jsonschema==4.17.0',
        'click==8.1.7',
        'loguru==0.7.0',
        'allure-pytest==2.9.45',
        'pytest==6.2.5',
        'pytest-xdist==2.5.0',
        'pytest-rerunfailures==10.2',
        'PyYAML==6.0.1'
    ],
    extras_require={
        "web": ['playwright==1.33.0'],
        "android": ['uiviewer',
                    'uiautomator2==2.16.23'],
        "ios": ['uiviewer',
                'tidevice==0.12.9',
                'facebook-wda==1.4.6'],
        "hm": ['uiviewer',
               'hmdriver2==1.3.0'],
    },
    entry_points={
        'console_scripts': [
            'kytest = kytest.cli:main'
        ]
    }
)



# 'pip install opencv-python==4.6.0.66'
import time

from .driver import ImageDiscern

from kytest.utils.log import logger
from kytest.utils.cv_util import draw_red_by_rect


class ImgElem(object):
    """图像识别定位"""

    def __init__(self,
                 driver=None,
                 file: str = None,
                 scale: int = None,
                 grade=0.9,
                 gauss_num=111,
                 _debug: bool = False):
        """

        @param driver: 设备驱动
        @param file: 图片文件路径
        @param scale: 用于IOS端缩放设置
        @param grade: 识别阈值设置
        @param gauss_num: 识别的次数
        @param _debug: 截图并圈选位置，用于调试
        """
        self.driver = driver
        self.target_image = file
        self._debug = _debug
        self._scale = scale
        self._grade = grade
        self._gauss_num = gauss_num

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def exists(self, timeout=1):
        logger.info(f'图像识别判断: {self.target_image} 是否存在')
        time.sleep(timeout)
        source_image = self.driver.shot("图像识别中")
        res = ImageDiscern(self.target_image,
                           source_image,
                           self._grade,
                           self._gauss_num).get_coordinate()
        logger.debug(res)
        if isinstance(res, tuple):
            return True
        else:
            return False

    def click(self, retry=3, timeout=3):
        logger.info(f'图像识别点击图片: {self.target_image}')
        for i in range(retry):
            time.sleep(timeout)
            logger.info(f'第{i + 1}次查找:')
            source_image = self.driver.shot("图像控件")
            res = ImageDiscern(self.target_image,
                               source_image,
                               self._grade,
                               self._gauss_num).get_coordinate()
            if isinstance(res, tuple):
                logger.info(f'识别坐标为: {res}')
                x, y = res[0], res[1]
                if self._scale is not None:
                    """iphone的scale是3"""
                    x, y = int(x/self._scale), int(y/self._scale)
                if self._debug is True:
                    file_path = self.driver.shot('图像识别定位成功')
                    if self._scale is not None:
                        _x, _y = x * self._scale, y * self._scale
                        draw_red_by_rect(file_path,
                                         (int(_x) - 100, int(_y) - 100, 200, 200))
                    draw_red_by_rect(file_path,
                                     (int(x) - 100, int(y) - 100, 200, 200))
                self.driver.click(x, y)
                return
        else:
            self.driver.shot(f'图像识别定位失败')
            raise Exception('未识别到图片，无法进行点击')


if __name__ == '__main__':
    pass





"""
@Author: kang.yang
@Date: 2023/9/20 11:21
"""
import time

from urllib import parse

from kytest.utils.config import kconfig
from kytest.utils.log import logger


class Page(object):
    """页面基类，用于pom模式封装"""

    def __init__(self, driver):
        self.driver = driver

    def open(self, url: str = None):
        if getattr(self, 'url', None) is None:
            if url is None:
                raise Exception('url不能为空')
        else:
            url = getattr(self, 'url')

        if not url.startswith('http'):
            web_host = kconfig["web_url"]
            if web_host:
                host = web_host
            else:
                host = kconfig["base_url"]
            if host is not None:
                url = parse.urljoin(host, url)
            else:
                raise Exception('host不能为空')

        self.driver.open(url)

    @staticmethod
    def sleep(n):
        logger.info(f"等待: {n}s")
        time.sleep(n)








"""
@Author: kang.yang
@Date: 2024/9/14 09:44
"""
from kytest.adr import Page, Elem


class AdrPage(Page):
    adBtn = Elem(resourceId='com.qizhidao.clientapp:id/bottom_btn')
    myTab = Elem(xpath='//android.widget.FrameLayout[4]')
    spaceTab = Elem(text='科创空间')
    setBtn = Elem(resourceId='com.qizhidao.clientapp:id/me_top_bar_setting_iv')
    title = Elem(resourceId='com.qizhidao.clientapp:id/tv_actionbar_title')
    agreeText = Elem(resourceId='com.qizhidao.clientapp:id/agreement_tv_2')
    moreService = Elem(xpath='//*[@resource-id="com.qizhidao.clientapp:id/layout_top_content"]'
                       '/android.view.ViewGroup[3]/android.view.View[10]')
    page_title = Elem(resourceId='com.qizhidao.clientapp:id/tv_actionbar_title')




from .running.runner import main
from .running.conf import App
from .utils.config import kconfig
from .utils.pytest_util import *
from .utils.allure_util import *
from .utils.log import logger
from .api import HttpReq, TestCase as TC

__version__ = "0.1.54"
__description__ = "API/安卓/IOS/WEB/鸿蒙Next平台自动化测试框架"



import shutil
import pytest

from multiprocessing import Process
from typing import Literal, Union

from kytest.running.conf import App
from kytest.utils.log import logger
from kytest.utils.config import kconfig
from kytest.running.execution_scheduler import (
    _get_case_list,
    _app_main
)


class TestMain(object):
    """
    测试框架入口
    """

    def __init__(
            self,
            title: str = None,
            path: str = None,
            host=None,
            headers: dict = None,
            pkg: str = None,
            did=None,
            ability=None,
            rule: Literal[
                "full_serial",
                "full_parallel",
                "split_serial",
                "split_parallel"] = 'full_serial',
            rerun: int = 0,
            xdist: Union[bool, int, str] = False
    ):
        """
        @param title：报告标题
        @param path: 用例路径
        @param host: 域名，用于接口测试和web测试
            如果host为str时，默认设置base_url
            如果要设置web_host，需要用{'web': 'https://www.qizhidao.com'}的方式传值
            如果接口和web都要使用，则设置{'api': '', 'web': ''}
        # @param web_host：web域名，用于web测试和接口测试并存时区分
        @param headers: 请求头，用于接口测试和web测试
        @param pkg:
            安卓包名，通过adb shell pm list packages | grep 'xxx'获取
            IOS包名，通过tidevice applist | grep 'xxx'获取
            鸿蒙包名，通过hdc shell aa dump -l 获取
        @param did:
            安卓设备id：通过adb devices获取
            IOS设备id：通过tidevice list获取
            鸿蒙设备id：通过hdc list targets获取
        @param ability: 鸿蒙应用的main name，通过如下命令获取
            $ hdc shell aa dump -l    # 运行命令前需要手动打开app
            User ID #100
              current mission lists:{
                Mission ID #139  mission name #[#com.kuaishou.hmapp:kwai:EntryAbility]  lockedState #0  mission affinity #[]
                  AbilityRecord ID #55
                    app name [com.kuaishou.hmapp]
                    main name [EntryAbility]
                    bundle name [com.kuaishou.hmapp]
                    ability type [PAGE]
                    state #FOREGROUND  start time [152523]
                    app state #FOREGROUND
                    ready #1  window attached #0  launcher #0
                    callee connections:
                    isKeepAlive: false
             }
        @param rule: 多设备执行模式
                - full_serial：每个设备都执行全部用例，串行执行
                - full_parallel：每个设备都执行全部用例，并发执行
                - split_serial：每个设备执行部分用例，串行执行
                - split_parallel：每个设备执行部分用例，并发执行
        @param rerun: 失败重试次数
        @param xdist: 是否并发执行，应该是多进程
        """
        logger.info("kytest start.")
        kconfig['project'] = title
        # kconfig['base_url'] = host
        # kconfig['web_url'] = web_host
        if isinstance(host, str):
            kconfig['base_url'] = host
        if isinstance(host, dict):
            kconfig['base_url'] = host.get('api', None)
            kconfig['web_url'] = host.get('web', None)
        kconfig['headers'] = headers

        App.did = did
        App.pkg = pkg
        App.ability = ability

        def _serial_execute(_path):
            # 串行执行用例
            cmd_list = [
                '-sv',
                '--reruns', str(rerun),
                '--alluredir', 'report', '--clean-alluredir'
            ]

            if _path:
                cmd_list.insert(0, _path)

            if xdist:
                if xdist is True:
                    cmd_list.insert(1, '-n')
                    cmd_list.insert(2, 'auto')
                    cmd_list.insert(3, '--dist=loadscope')
                elif isinstance(xdist, int) or isinstance(xdist, str):
                    cmd_list.insert(1, '-n')
                    cmd_list.insert(2, str(xdist))
                    cmd_list.insert(3, '--dist=loadscope')

            logger.info(cmd_list)
            pytest.main(cmd_list)

        def _parallel_execute(_params):
            # 并发执行用例
            if 'parallel' in rule:
                if _params:
                    for param in _params:
                        pr = Process(target=_app_main, args=param)
                        pr.start()
            elif 'serial' in rule:
                if _params:
                    for param in _params:
                        _app_main(*param)

        def _collect_case_and_split(device_id, _path):
            # 用例采集
            _path_list = [{item: []} for item in device_id]
            test_cases = _get_case_list(_path)
            print(test_cases)
            # 把用例均分成设备数量的份数
            n = len(device_id)
            _lists = [[] for _ in range(n)]
            for _i, item in enumerate(test_cases):
                index = _i % n  # 计算元素应该分配给哪个列表
                _lists[index].append(item)
            return _lists

        # 多设备执行策略
        if isinstance(did, list):
            params = []
            if not did:
                _serial_execute(path)
            elif len(did) == 1:
                App.did = did[0]
                _serial_execute(path)
            else:
                # 清空上次执行的目录
                shutil.rmtree("report", ignore_errors=True)
                if 'full' in rule:
                    for device in did:
                        params.append((path, device, pkg, True))
                else:
                    lists = _collect_case_and_split(did, path)

                    for i in range(len(did)):
                        _path = lists[i][0] if len(lists[1]) < 2 else ','.join(lists[i])
                        params.append((_path, did[i], pkg))

                # 多进程执行
                _parallel_execute(params)

        else:
            # 串行执行
            _serial_execute(path)

        # 公共参数重置
        kconfig.reset()
        # App参数重置
        App.did = None
        App.pkg = None


main = TestMain

if __name__ == '__main__':
    main()



"""
@Author: kang.yang
@Date: 2023/10/26 09:48
"""
import time

import allure
from urllib import parse

from .driver import Driver
from .element import Elem

from kytest.api.request import HttpReq
from kytest.utils.log import logger
from kytest.utils.config import kconfig
from kytest.web.config import BrowserConfig


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    dr: Driver = None

    # ---------------------初始化-------------------------------
    @classmethod
    def start_class(cls):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    @classmethod
    def end_class(cls):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        # 使用kconfig进行设置属于进阶功能，并发执行的时候才用，一般用BrowserConfig就行了。
        if kconfig["browser"]:
            browserName = kconfig["browser"]
        else:
            browserName = BrowserConfig.browser_name
        if kconfig["headless"]:
            headless = kconfig["headless"]
            headless = True if headless else False
        else:
            headless = BrowserConfig.headless
        if kconfig["full"]:
            maximized = kconfig["full"]
            maximized = True if maximized else False
        else:
            maximized = BrowserConfig.maximized
        if kconfig["size"]:
            window_size = kconfig["size"]
        else:
            window_size = BrowserConfig.window_size
        if kconfig["state"]:
            state_file = kconfig["state"]
        else:
            state_file = BrowserConfig.state
        # if not state_file:
        #     try:
        #         cookies = kconfig["cookies"]
        #         if cookies:
        #             if not os.path.exists("data"):
        #                 os.makedirs("data")
        #             state_file = os.path.join("data", "state.json")
        #             state_json = {
        #                 "cookies": cookies
        #             }
        #             with open(state_file, "w") as f:
        #                 f.write(json.dumps(state_json))
        #     except Exception as e:
        #         print(e)
        #         state_file = None

        cls.dr = Driver(
            browserName=browserName,
            headless=headless,
            state=state_file,
            maximized=maximized,
            window_size=window_size
        )
        cls.context = cls.dr.context
        cls.page = cls.dr.page

        cls.start_class()

    @classmethod
    def teardown_class(cls):
        cls.end_class()
        cls.dr.close()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        project_name = kconfig['project']
        if project_name:
            allure.dynamic.feature(project_name)

        self.start()

    def teardown_method(self):
        self.end()

    # 公共方法
    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def shot(self, name: str, delay=0):
        """截图"""
        if delay:
            self.sleep(delay)
        self.dr.shot(name)

    # UI方法
    def elem(self, xpath=None, css=None):
        _kwargs = {}
        if xpath is not None:
            _kwargs['xpath'] = xpath
        if css is not None:
            _kwargs['css'] = css

        return Elem(self.dr, **_kwargs)

    # web方法
    def assert_title(self, title: str, timeout: int = 5):
        """断言页面标题"""
        self.dr.assert_title(title, timeout)

    @staticmethod
    def is_url_has_http(url):
        """针对为空和只有路径的情况，使用默认host进行补全"""
        web_host = kconfig["web_url"]
        if web_host:
            host = web_host
        else:
            host = kconfig["base_url"]
        if url is None:
            url = host
        if 'http' not in url:
            url = parse.urljoin(host, url)
        return url

    def assert_url(self, url: str = None, timeout: int = 10):
        """断言页面url"""
        url = self.is_url_has_http(url)
        self.dr.assert_url(url, timeout)

    def goto(self, url):
        """打开页面"""
        url = self.is_url_has_http(url)
        self.dr.goto(url)





"""
@Author: kang.yang
@Date: 2024/7/12 17:10
"""
import kytest
from kytest.web import BrowserConfig

from data.login_data import get_headers


if __name__ == '__main__':
    # 不常用的浏览器参数设置
    BrowserConfig.headless = True

    hosts = {
        'api': 'https://app-test.qizhidao.com/',
        'web': 'https://www-test.qizhidao.com/'
    }
    kytest.main(
        pkg="com.qizhidao.clientapp",  # 应用包名，针对IOS、安卓、鸿蒙
        ability="EntryAbility",  # 页面名，针对鸿蒙，启动应用时使用
        host=hosts,  # 域名，针对接口和web测试
        headers=get_headers(),  # 请求头信息，针对接口测试
        path="tests/test_web.py",  # 测试脚本目录
        # xdist=True  # 并发执行，针对接口和web测试tests/Product_Detail_Controller
    )




"""
@Author: kang.yang
@Date: 2024/10/8 15:12
"""
from kytest.hm import TestCase

from page.hm_page import HmPage


class TestHmDemo(TestCase):

    def start(self):
        self.start_app()
        self.hm = HmPage(self.dr)

    def test_hm(self):
        self.dr.click(630, 2050)
        self.hm.my_entry.click()
        self.hm.login_entry.click()
        assert self.hm.pwd_login.exists()



